/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.bridge;

import static org.wso2.carbon.bridge.FrameworlLauncherConstants.MANIFEST_VERSION;
import static org.wso2.carbon.bridge.FrameworlLauncherConstants.BUNDLE_MANIFEST_VERSION;
import static org.wso2.carbon.bridge.FrameworlLauncherConstants.BUNDLE_NAME;
import static org.wso2.carbon.bridge.FrameworlLauncherConstants.BUNDLE_SYMBOLIC_NAME;
import static org.wso2.carbon.bridge.FrameworlLauncherConstants.BUNDLE_VERSION;
import static org.wso2.carbon.bridge.FrameworlLauncherConstants.FRAGMENT_HOST;
import static org.wso2.carbon.bridge.FrameworlLauncherConstants.BUNDLE_CLASSPATH;

import org.osgi.framework.Constants;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import java.io.*;
import java.util.*;
import java.util.jar.Manifest;
import java.util.jar.Attributes;
import java.util.jar.JarOutputStream;
import java.util.jar.JarEntry;

public class KnopflerfishFrameworkLauncher implements FrameworkLauncher {

    protected static final String RESOURCE_BASE = "/WEB-INF/knopflerfish/";

    protected ServletConfig config;
    protected ServletContext context;
    private File platformDirectory;
    private File fwdirDirectory;
    private File pluginsDirectory;
    private File configurationBase;

    private ClassLoader frameworkContextClassLoader;

    // Set to true if JVM is started without any arguments
    static boolean bZeroArgs     /*     = false*/;

    // Top directory (including any trailing /)
    static String topDir = "";

    // Will be filled in from manifest entry during startup
    static String version = "<unknown>";

    static long bootMgr/* = 0*/;

    // Xargs to use for FW init
    static String defaultXArgsInit = "init.xargs";

    // Xargs to use for FW init
    static final String defaultXArgsInit2 = "remote-init.xargs";

    // Xargs to use for FW restart
    static final String defaultXArgsStart = "restart.xargs";

    public static boolean bWriteSysProps = true;

    public static final String FWDIR_PROP = "org.osgi.framework.dir";
    public static final String PRODVERSION_PROP = "org.knopflerfish.prodver";
    public static final String JARDIR_PROP = "org.knopflerfish.gosg.jars";
    public static final String JARDIR_DEFAULT = "file:";


    public String FWDIR_DEFAULT = "fwdir";


    public void init(ServletConfig servletConfig) {
        config = servletConfig;
        context = servletConfig.getServletContext();
    }

    public void start() {

        fwdirDirectory = new File(platformDirectory, FWDIR_DEFAULT);
        FWDIR_DEFAULT = fwdirDirectory.getAbsolutePath();
        
        System.setProperty(FWDIR_PROP, FWDIR_DEFAULT);
        System.setProperty(JARDIR_PROP, "file:" + pluginsDirectory.getAbsolutePath() + File.separator);

        File xargsFile = new File(configurationBase, defaultXArgsInit);

        String [] args = new String[]{"-xargs", xargsFile.getAbsolutePath()};


        String servletVersion =
            context.getMajorVersion() + "." + context.getMinorVersion();

        String[][] defaultSysProps = new String[][]{
                {Constants.FRAMEWORK_SYSTEMPACKAGES, "javax.accessibility,javax.net,javax.net.ssl," +
                        "org.wso2.carbon.bridge; version=1.0.0, org.wso2.carbon.server.transports; version=1.0.0," +
                        "javax.servlet; version=" + servletVersion + ", javax.servlet.http; version=" + servletVersion +
                        ", javax.servlet.resources; version=" +servletVersion }

        };

        System.setProperty(defaultSysProps[0][0], defaultSysProps[0][1]);


//        Method 1 - Here preCL can be any class loader..
//        ClassLoader preCL = this.getClass().getClassLoader().getParent().getParent();
//        try{
//            Class mainClass = preCL.loadClass("org.knopflerfish.framework.Main");
//
//            Method mainMethod = mainClass.getMethod("main", new Class[]{String[].class});
//            mainMethod.invoke(null, new Object[]{args});
//
//        }catch(Exception e){
//            e.printStackTrace();
//        }

//        Method 2
//        Main.main(args);




        ClassLoader prevCls = Thread.currentThread().getContextClassLoader();
            try {
                ClassLoader newCls = new ContextFinder(prevCls);
                Thread.currentThread().setContextClassLoader(newCls);
                frameworkContextClassLoader = newCls;
// TODO Upload the Knopflerfish jars to a maven repo..
//                Main.main(args);
            } finally {
                Thread.currentThread().setContextClassLoader(prevCls);
            }
    }

    public void stop() {
    }

    public void deploy() {
        if (platformDirectory != null) {
            context.log("Framework is already deployed");
            return;
        }

        File servletTemp =
                (File) context.getAttribute("javax.servlet.context.tempdir");
        platformDirectory = new File(servletTemp, "knopflerfish");
        if (!platformDirectory.exists()) {
            platformDirectory.mkdirs();
        }

        configurationBase = new File(platformDirectory, "configuration");
        copyResource(RESOURCE_BASE + "configuration/", configurationBase);
        pluginsDirectory = new File(platformDirectory, "plugins");
        copyResource(RESOURCE_BASE + "plugins/", pluginsDirectory);
//        deployDocumentationFragmentHost(pluginsDirectory);
        copyResource("/WEB-INF/plugins/", pluginsDirectory);
        BundleExtensionProvider.provideExtensions(pluginsDirectory);
    }

    public void undeploy() {
    }

    public ClassLoader getFrameworkContextClassLoader() {
        return frameworkContextClassLoader;
    }

    private void deployDocumentationFragmentHost(File plugins) {
        File docsFragmentHost = new File(plugins,
                "org.wso2.carbon.documentation.extensionbundle_1.0.0.jar");
        if (docsFragmentHost.exists())
            return;

        String carbonHome = System.getProperty("carbon.home");
        if (carbonHome == null || carbonHome.length() == 0) {
            return;
        }

        File docsDir = new File(carbonHome + File.separator + "docs");
        File webDir = new File(plugins, "web");
        if (!docsDir.exists())
            return;

        copyResource(docsDir.getAbsolutePath() + File.separator, webDir);

        Manifest mf = new Manifest();
        Attributes attribs = mf.getMainAttributes();
        attribs.putValue(MANIFEST_VERSION, "1.0");
        attribs.putValue(BUNDLE_MANIFEST_VERSION, "2");
        attribs.putValue(BUNDLE_NAME, "Docs Extension Bundle");
        attribs.putValue(BUNDLE_SYMBOLIC_NAME,
                "org.wso2.carbon.documentation.extensionbundle");
        attribs.putValue(BUNDLE_VERSION, "1.0.0");
        attribs.putValue(FRAGMENT_HOST, "org.wso2.carbon.documentation");
        attribs.putValue(BUNDLE_CLASSPATH, ".");

        try {
            JarOutputStream jos = new JarOutputStream(new FileOutputStream(docsFragmentHost), mf);
            String baseDirPath = plugins.getPath() + File.separator;
            int baseDirPathLength = baseDirPath.length();
            checkDirectory(webDir.getPath(), jos, baseDirPathLength);
            try {

                jos.finish();
            } finally {
                if (jos != null)
                    jos.close();
            }
        } catch (IOException e) {
            context.log("Error generating extension bundle", e);

        } finally {
            webDir.delete();
        }
    }

    /**
     * copyResource is a convenience method to recursively copy resources from the ServletContext to
     * an installation target. The default behaviour will create a directory if the resourcepath ends
     * in '/' and a file otherwise.
     *
     * @param resourcePath - The resource root path
     * @param target       - The root location where resources are to be copied
     */
    protected void copyResource(String resourcePath, File target) {
        if (resourcePath.endsWith("/")) {
            target.mkdir();
            Set paths = context.getResourcePaths(resourcePath);
            if (paths == null)
                return;
            for (Iterator it = paths.iterator(); it.hasNext();) {
                String path = (String) it.next();
                File newFile = new File(target, path.substring(resourcePath.length()));
                copyResource(path, newFile);
            }
        } else {
            try {
                if (target.createNewFile()) {
                    InputStream is = null;
                    OutputStream os = null;
                    try {
                        is = context.getResourceAsStream(resourcePath);
                        if (is == null)
                            return;
                        os = new FileOutputStream(target);
                        byte[] buffer = new byte[8192];
                        int bytesRead = is.read(buffer);
                        while (bytesRead != -1) {
                            os.write(buffer, 0, bytesRead);
                            bytesRead = is.read(buffer);
                        }
                    } finally {
                        if (is != null)
                            is.close();

                        if (os != null)
                            os.close();
                    }
                }
            } catch (IOException e) {
                context.log("Error copying resources", e);
            }
        }
    }

    public void checkDirectory(String directoryName, JarOutputStream jos, int baseDirPathLength) {
        File dirobject = new File(directoryName);
        if (dirobject.exists() == true) {
            if (dirobject.isDirectory() == true) {
                File[] fileList = dirobject.listFiles();
                // Loop through the files
                for (int i = 0; i < fileList.length; i++) {
                    if (fileList[i].isDirectory()) {
                        checkDirectory(fileList[i].getPath(), jos, baseDirPathLength);
                    } else if (fileList[i].isFile()) {
                        // Call the zipFunc function
                        jarFile(fileList[i].getPath(), jos, baseDirPathLength);
                    }
                }
            } else {
                context.log(directoryName + " is not a directory.");
            }
        } else {
            context.log("Directory " + directoryName + " does not exist.");
        }
    }

    private void jarFile(String filePath, JarOutputStream jos, int baseDirPathLength) {
        try {
            FileInputStream fis = new FileInputStream(filePath);
            BufferedInputStream bis = new BufferedInputStream(fis);
            JarEntry fileEntry = new JarEntry(filePath.substring(baseDirPathLength));
            jos.putNextEntry(fileEntry);
            byte[] data = new byte[1024];
            int byteCount;
            while ((byteCount = bis.read(data, 0, 1024)) > -1) {
                jos.write(data, 0, byteCount);
            }
        } catch (IOException e) {
        }
    }
}