/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.core;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.core.persistence.PersistenceManager;
import org.wso2.carbon.utils.ServerConfiguration;
import org.wso2.carbon.utils.ServerException;

import java.util.Iterator;

/*
 * The default initializer which initilizes WSAS
 */
public class DefaultServerInitializer implements ServerInitializer {
    private static final Log log = LogFactory.getLog(DefaultServerInitializer.class);

    private PersistenceManager pm;
    private ServerManager serverManager;

    public DefaultServerInitializer() {
        ServerConfiguration serverConfig = ServerConfiguration.getInstance();
        this.serverManager = ServerManager.getInstance();
    }


    public void init(ConfigurationContext configurationContext) throws AxisFault, ServerException {

        // Enabling http binding generation
        Parameter enableHttp = new Parameter("enableHTTP", "true");
        AxisConfiguration axisConfig = configurationContext.getAxisConfiguration();
        axisConfig.addParameter(enableHttp);

        this.pm = new PersistenceManager(axisConfig);

        persistTransports(axisConfig);

        System.out.println();
        log.info("Repository             : " + serverManager.axis2RepoLocation);
    }

    private void persistTransports(AxisConfiguration axisConfig) {

        //Check the associations.
//        // TODO Check whether the user has removed transports from Axis2.xml
//        TransportDO[] transports = pm.getTransports();
//        for (int i = 0; i < transports.length; i++) {
//            TransportDO transport = transports[i];
//            if (axisConfig.
//                    getTransportIn(transport.getTransportProtocol()) == null) {
//                if (!transport.getServices().isEmpty()) {
//                    String msg = "Cannot remove the " + transport.getTransportProtocol() + " transport.\n" +
//                                 "\t\t\t\tOne or more services are bound to this transport. \n" +
//                                 "\t\t\t\tAdd this transport back to the axis2.xml file and restart the server.";
//                    throw new AxisFault(msg);
//                } else {
//                    pm.deleteEntity(transport);
//                }
//            }
//        }

        // Persist the transport protocols
        try {
			for (Iterator transportsIter = axisConfig.getTransportsIn().values().iterator(); transportsIter
					.hasNext();) {
				TransportInDescription tiDesc = (TransportInDescription) transportsIter.next();
				String protocolName = tiDesc.getName();
				pm.addTransportOnStartup(protocolName, true, tiDesc.getParameters());

			}
		} catch (Exception e) {

		}

// PersistenceManager pm = new PersistenceManager();
//
//        // Check whether the user has removed transports from Axis2.xml
//        TransportDO[] transports = pm.getTransports();
//        for (TransportDO transport : transports) {
//            if (axisConfig.
//                    getTransportIn(transport.getTransportProtocol()) == null) {
//                if (!transport.getServices().isEmpty()) {
//                    String msg = "Cannot remove the " + transport.getTransportProtocol() + " transport.\n" +
//                                 "\t\t\t\tOne or more services are bound to this transport. \n" +
//                                 "\t\t\t\tAdd this transport back to the axis2.xml file and restart the server.";
//                    throw new AxisFault(msg);
//                } else {
//                    pm.deleteEntity(transport);
//                }
//            }
//        }
//
//        // Persist the transport protocols
//        for (Iterator transportsIter = axisConfig.getTransportsIn().values().iterator();
//             transportsIter.hasNext();) {
//            TransportInDescription tiDesc = (TransportInDescription) transportsIter.next();
//            String protocolName = tiDesc.getName();
//            if (pm.getTransport(protocolName) == null) { // Transport does not already exist?
//                TransportDO transportDO = new TransportDO();
//                transportDO.setTransportProtocol(protocolName);
//                transportDO.setIsStarted(true);
//                try {
//                    pm.addTransport(transportDO);
//                } catch (TransportAlreadyExistsException ignored) {
//                    // ignored
//                }
//            }
//        }
    }
}
