/*
 * Copyright 2005-2008 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.core.transports;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.llom.util.AXIOMUtil;
import org.apache.axis2.AxisFault;
import org.apache.axis2.deployment.DeploymentConstants;
import org.apache.axis2.deployment.util.Utils;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.util.XMLUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.core.persistence.PersistenceManager;
import org.wso2.carbon.core.transports.util.TransportDetails;
import org.wso2.carbon.core.transports.util.TransportParameter;
import org.wso2.carbon.core.transports.util.TransportSummary;
import org.wso2.carbon.registry.core.Resource;
import org.wso2.carbon.utils.ServerConstants;

/**
 * 
 * 
 */
public class CommonTransportLoader implements DeploymentConstants {

	private AxisConfiguration axisConfiguration = null;
	private String configFile = null;

	private final String TRANSPORT_RECEIVER = "transportReceiver";
	private static Log log = LogFactory.getLog(CommonTransportLoader.class);

	/**
	 * 
	 * @param config
	 */
	public CommonTransportLoader(AxisConfiguration config) {
		axisConfiguration = config;
	}

	public CommonTransportLoader(AxisConfiguration config, String configFile) {
		axisConfiguration = config;
		this.configFile = System.getProperty(ServerConstants.CARBON_HOME) + File.separator + "conf"
				+ File.separator + configFile;
	}

	/**
	 * 
	 * @param transport
	 * @return
	 * @throws AxisFault
	 */
	public Map<String, String> loadTransportSettings(String transport) throws AxisFault {
		TransportDetails details = null;
		TransportParameter[] transportParams = null;
		Map<String, String> params = null;

		details = getTransportDetails(transport);
		params = new HashMap<String, String>();

		if (details != null) {
			transportParams = details.getParameters();
			for (int i = 0; i < transportParams.length; i++) {
				TransportParameter transportPara = transportParams[i];
				params.put(transportPara.getName(), transportPara.getValue());
			}
		}

		return params;
	}

	/**
	 * 
	 * @param transport
	 * @return
	 * @throws AxisFault
	 */
	public boolean isActive(String transport) throws AxisFault {
		if (axisConfiguration.getAxisConfiguration().getTransportIn(transport) != null) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * 
	 * @throws AxisFault
	 */
	public void enableTransport() throws AxisFault {
		Map services = axisConfiguration.getServices();
		Iterator serviceNames = services.keySet().iterator();

		while (serviceNames.hasNext()) {
			String serviceName = (String) serviceNames.next();
			AxisService axisService = (AxisService) axisConfiguration
					.getServiceForActivation(serviceName);

			// We do not need to worry about admin services.
			if (axisService.getParameter("adminService") == null
					|| !"true".equals(axisService.getParameter("adminService").getValue()
							.toString())) {
				Utils.addEndpointsToService(axisService, axisConfiguration);
			}
		}
	}

	/**
	 * 
	 * @return
	 */
	public TransportSummary[] listTransports() {
		HashMap transports = null;
		Collection transCollection = null;

		transports = getAxisConfig().getTransportsIn();
		transCollection = new ArrayList();

		for (Iterator iter = transports.values().iterator(); iter.hasNext();) {
			TransportInDescription inTransport = null;
			TransportSummary summary = null;

			inTransport = (TransportInDescription) iter.next();
			summary = new TransportSummary();
			summary.setProtocol(inTransport.getName());
			summary.setActive(true);
			transCollection.add(summary);
		}

		return (TransportSummary[]) transCollection.toArray(new TransportSummary[transCollection
				.size()]);
	}

	/**
	 * 
	 * @param transportProtocol
	 * @return
	 * @throws AxisFault
	 */
	public TransportDetails getTransportDetails(String transportProtocol) throws AxisFault {
		TransportInDescription inTransport = null;

		inTransport = getAxisConfig().getTransportIn(transportProtocol);

		if (inTransport == null) {
			if (configFile != null) {
				return getTransportDetails(readTransportConfiguration());
			} else {
				return null;
			}
		} else {
			return getTransportDetails(inTransport);
		}
	}

	/**
	 * 
	 * @param serviceName
	 * @param transportProtocol
	 * @return
	 * @throws AxisFault
	 */
	public TransportDetails getTransportDetails(String serviceName, String transportProtocol)
			throws AxisFault {
		TransportInDescription inTransport = null;
		AxisService service = null;

		service = getAxisConfig().getService(serviceName);
		inTransport = service.getAxisConfiguration().getTransportIn(transportProtocol);

		if (inTransport == null) {
			if (configFile != null) {
				return getTransportDetails(readTransportConfiguration());
			} else {
				return null;
			}
		} else {
			return getTransportDetails(inTransport);
		}
	}

	/**
	 * 
	 * @return
	 */
	public TransportInDescription readTransportConfiguration() {
		FileInputStream config = null;
		OMElement element = null;
		TransportBuilder transportBuilder = null;
		Iterator transportReivers = null;
		TransportInDescription transportIn = null;
		TransportInDescription[] transportIns = null;
		String name = null;
		ArrayList params = null;
		Resource resource = null;
		Resource resourceElement = null;
		PersistenceManager pm = null;

		if (configFile == null) {
			return null;
		}

		try {
			try {
				config = new FileInputStream(configFile);
			} catch (FileNotFoundException e) {
				if (log.isDebugEnabled()) {
					log.error("Transport configuration filr not found: " + configFile, e);
				}
				// Configuration file not found in the given location - this transport wont be
				// available for enabling/disabling.
				return null;
			}
			element = (OMElement) XMLUtils.toOM(config);
			element.build();

			// Processing Transport Receivers
			transportReivers = element.getChildrenWithName(new QName(TRANSPORT_RECEIVER));
			transportBuilder = new TransportBuilder();

			transportIns = transportBuilder.processTransportReceivers(transportReivers);

			if (transportIns != null && transportIns.length > 0) {
				transportIn = transportIns[0];
			}
			if (transportIn == null) {
				return null;
			}

			params = transportIn.getParameters();
			name = transportIn.getName();
			pm = new PersistenceManager(axisConfiguration);

			for (Iterator<Parameter> paramIterator = params.iterator(); paramIterator.hasNext();) {
				Parameter parameter = paramIterator.next();

				String key = parameter.getName();
				String content = null;
				String elementContent = null;
				Parameter param = null;

				resource = pm.getTransportParam(name, key);
				resourceElement = pm.getTransportParam(name, key + ".element");

				// We need to attend this later - right now we don't let
				// users edit the port. So we read it from base
				// configuration.
				if (!"port".equals(key)) {
					param = transportIn.getParameter(key);
					if (resource != null) {
						content = new String((byte[]) resource.getContent());
						try {
							param.setValue(AXIOMUtil.stringToOM(content));
						} catch (Exception e) {
							param.setValue(content);
						}
					}
					if (resourceElement != null) {
						elementContent = new String((byte[]) resourceElement.getContent());
						param.setParameterElement(AXIOMUtil.stringToOM(elementContent));
					}
				}
				if (resourceElement != null) {
					resourceElement.discard();
				}
				if (resource != null) {
					resource.discard();
				}
			}
		} catch (Exception e) {
			log.error("Error while reading transport configurations", e);
		}

		return transportIn;
	}

	/**
	 * 
	 * @param inTransport
	 * @return
	 */
	private TransportDetails getTransportDetails(TransportInDescription inTransport) {
		TransportDetails details = null;
		List params = null;
		int intValue = -1;

		if (inTransport == null) {
			return null;
		}

		details = new TransportDetails();
		params = new ArrayList();
		details.setActive(true);

		// Transport parameters
		for (Iterator paramIter = inTransport.getParameters().iterator(); paramIter.hasNext();) {
			Parameter param = null;
			TransportParameter paramDO = null;
			OMElement paramEle = null;

			param = (Parameter) paramIter.next();
			paramDO = new TransportParameter();
			paramDO.setName(param.getName());
			paramEle = param.getParameterElement();

			try {
				if (param.getValue() != null) {
					intValue = Integer.parseInt(param.getValue().toString());
				}
			} catch (NumberFormatException e) {
				intValue = 0;
			}

			// We need to avoid invalid values.
			if (intValue >= 0) {
				if (paramEle != null) {
					paramDO.setParamElement(paramEle.toString());
				}
				paramDO.setValue(param.getValue().toString());
				params.add(paramDO);
			}
		}

		details.setParameters((TransportParameter[]) params.toArray(new TransportParameter[params
				.size()]));
		return details;

	}

	/**
	 * 
	 * @param servieName
	 * @param transportProtocol
	 * @param params
	 * @throws AxisFault
	 */
	public void updateTransportDetails(String serviceName, String transportProtocol,
			TransportParameter[] params) throws AxisFault {
		AxisService service = null;
		TransportInDescription inTransport = null;

		service = getAxisConfig().getService(serviceName);
		inTransport = service.getAxisConfiguration().getTransportIn(transportProtocol);

		if (inTransport == null) {
			// We have no transports defined for this.
			TransportInDescription transport = null;
			transport = readTransportConfiguration();
			if (transport != null) {
				getAxisConfig().addTransportIn(transport);
				inTransport = getAxisConfig().getTransportIn(transportProtocol);
				updateTransportDetails(inTransport, params);
			}
		} else {
			updateTransportDetails(inTransport, params);
		}
	}

	/**
	 * 
	 * @param transportProtocol
	 * @param params
	 * @throws AxisFault
	 */
	public void updateTransportDetails(String transportProtocol, TransportParameter[] params)
			throws AxisFault {
		TransportInDescription inTransport = null;

		inTransport = getAxisConfig().getTransportIn(transportProtocol);

		if (inTransport == null) {
			// We have no transports defined for this.
			TransportInDescription transport = null;
			transport = readTransportConfiguration();
			if (transport != null) {
				getAxisConfig().addTransportIn(transport);
				inTransport = getAxisConfig().getTransportIn(transportProtocol);
				updateTransportDetails(inTransport, params);
			}
		} else {
			updateTransportDetails(inTransport, params);
		}
	}

	/**
	 * 
	 * @param inTransport
	 * @param params
	 * @throws AxisFault
	 */
	protected void updateTransportDetails(TransportInDescription inTransport,
			TransportParameter[] transParams) throws AxisFault {
		Parameter param = null;
		TransportParameter transportParameter = null;

		for (int i = 0; i < transParams.length; i++) {
			transportParameter = transParams[i];
			if (transportParameter != null && transportParameter.getName() != null) {
				param = inTransport.getParameter(transportParameter.getName());
				if (param != null && !param.isLocked()) {
					try {
						if (transportParameter.getParamElement() != null) {
							param.setParameterElement(AXIOMUtil.stringToOM(transportParameter
									.getParamElement()));
						}
					} catch (XMLStreamException e) {
						throw new AxisFault("Error updating transport details", e);
					}
					param.setValue(transportParameter.getValue());
				}
			}
		}
	}

	private AxisConfiguration getAxisConfig() {
		return axisConfiguration;
	}
}