/*
 * Copyright (c) 2008, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.carbon.registry.core.jdbc.realm;

import org.wso2.carbon.registry.core.RegistryConstants;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import java.util.Map;
import java.util.List;

public class RegistryUserStoreAdmin implements UserStoreAdmin {

    private static final Log log = LogFactory.getLog(RegistryUserStoreAdmin.class);

    private UserRealm coreRealm;

    public RegistryUserStoreAdmin(UserRealm coreRealm) {
        this.coreRealm = coreRealm;
    }

    //public RegistryUserStoreAdmin(DataSource dataSource) throws UserStoreException {
    //    super(dataSource);
    //}

    public void addUser(String userName, Object credential) throws UserStoreException {
        getUserStoreAdmin().addUser(userName, credential);
        addUserToRole(userName, RegistryConstants.EVERYONE_ROLE);
    }

    public void deleteUser(String userName) throws UserStoreException {

        if (userName.equals(RegistryConstants.SYSTEM_USER) ||
            userName.equals(RegistryConstants.ADMIN_USER) ||
            userName.equals(RegistryConstants.ANONYMOUS_USER)) {

            String msg = "Could not remove the system defined user: " + userName;
            throw new UserStoreException(msg, false);
        }

        getUserStoreAdmin().deleteUser(userName);
    }


    public void deleteRole(String roleName) throws UserStoreException {

        if (roleName.equals(RegistryConstants.ADMIN_ROLE) ||
            roleName.equals(RegistryConstants.EVERYONE_ROLE) ||
            roleName.equals(RegistryConstants.GUESTS_ROLE)) {

            String msg = "Could not remove the system defined role: " + roleName;
            throw new UserStoreException(msg, false);
        }

        getUserStoreAdmin().deleteRole(roleName);
    }


    public void removeUserFromRole(String userName, String roleName) throws UserStoreException {

        if (roleName.equals(RegistryConstants.EVERYONE_ROLE)) {

            String msg = "Could not remove the everyone role from the user: " +
                         userName + ". All users belong to the everyone role.";
            throw new UserStoreException(msg, false);
        }

        getUserStoreAdmin().removeUserFromRole(userName, roleName);
    }


    public void updateUser(String userName, Object newCredential, Object oldCredential)
            throws UserStoreException {

        // we should not allow to edit anoymous or system users.
        if (RegistryConstants.SYSTEM_USER.equals(userName) ||
                RegistryConstants.ANONYMOUS_USER.equals(userName)) {

            String msg = "Could not edit the system defined user: " + userName;
            throw new UserStoreException(msg);
        }

        getUserStoreAdmin().updateUser(userName, newCredential, oldCredential);
    }

    public String[] listUsers(String filter, int maxItemLimit) throws UserStoreException{
	return new String[0];
    }

    public void updateUser(String userName, Object newCredential) throws UserStoreException {
        getUserStoreAdmin().updateUser(userName, newCredential);
    }

    public void setUserProperties(String userName, Map properties) throws UserStoreException {
        getUserStoreAdmin().setUserProperties(userName, properties);
    }

    public void addRole(String roleName) throws UserStoreException {
        getUserStoreAdmin().addRole(roleName);
    }

    public void setRoleProperties(String roleName, Map properties) throws UserStoreException {
        getUserStoreAdmin().setRoleProperties(roleName, properties);
    }

    public void addUserToRole(String userName, String roleName) throws UserStoreException {
        getUserStoreAdmin().addUserToRole(userName, roleName);
    }

    public String[] getAllUserNames() throws UserStoreException {
        return getUserStoreAdmin().getAllUserNames();
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        return getUserStoreAdmin().isExistingUser(userName);
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        return getUserStoreAdmin().isExistingRole(roleName);
    }

    public Map getUserProperties(String userName) throws UserStoreException {
        return getUserStoreAdmin().getUserProperties(userName);
    }

    public String[] getUserPropertyNames() throws UserStoreException {
        return getUserStoreAdmin().getUserPropertyNames();
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        return getUserStoreAdmin().getUserNamesWithPropertyValue(propertyName, propetyValue);
    }

    public String[] getAllRoleNames() throws UserStoreException {
        return getUserStoreAdmin().getAllRoleNames();
    }

    public Map getRoleProperties(String roleName) throws UserStoreException {
        return getUserStoreAdmin().getRoleProperties(roleName);
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
        return getUserStoreAdmin().getUserRoles(userName);
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        return getUserStoreAdmin().getUsersInRole(roleName);
    }

    public boolean isExistingUserProfile(String s, String s1) throws UserStoreException {
        return false;
    }

    public List<String> getUserProfileNames(String s) throws UserStoreException {
        return null;
    }

    public String getDefaultUserProfileName(String s) throws UserStoreException {
        return null;
    }

    public void setUserProperties(String s, Map<String, String> stringStringMap, String s1)
            throws UserStoreException {
    }

    public void updateUserProperties(String s, Map<String, String> stringStringMap, String s1)
            throws UserStoreException {
    }

    public Map<String, String> getUserProperties(String s, String s1) throws UserStoreException {
        return null;
    }

    public void updateRoleName(String roleName, String newRoleName) throws UserStoreException{
    }
    
    public void updateUserName(String userName, String newUserName) throws UserStoreException{
	
    }
    
    private UserStoreAdmin getUserStoreAdmin() throws org.wso2.carbon.user.core.UserStoreException {
        return coreRealm.getUserStoreAdmin();
    }
    
}
