
CREATE TABLE REG_CLUSTER_LOCK (
                    REG_LOCK_NAME VARCHAR (20),
                    REG_LOCK_STATUS VARCHAR (20),
                    REG_LOCKED_TIME TIMESTAMP,
                    PRIMARY KEY (REG_LOCK_NAME))
/
CREATE TABLE REG_CONTENT (
                    REG_CONTENT_ID VARCHAR2 (50),
                    REG_CONTENT_DATA BLOB,
                    PRIMARY KEY (REG_CONTENT_ID))
/
CREATE TABLE REG_RESOURCE (
                    REG_RID VARCHAR2 (50),
                    REG_PATH VARCHAR2 (2000) NOT NULL,
                    REG_MEDIA_TYPE VARCHAR2 (500),
                    REG_COLLECTION INTEGER NOT NULL,
                    REG_CREATOR VARCHAR2 (500),
                    REG_CREATED_TIME TIMESTAMP,
                    REG_LAST_UPDATOR VARCHAR2 (500),
                    REG_LAST_UPDATED_TIME TIMESTAMP,
                    REG_DESCRIPTION VARCHAR2 (4000),
                    REG_CONTENT_ID VARCHAR2 (50),
                    REG_EQUIVALENT_VERSION INTEGER NOT NULL,
                    REG_ASSOCIATED_SNAPSHOT_ID INTEGER NOT NULL,
                    PRIMARY KEY (REG_RID),
                    FOREIGN KEY (REG_CONTENT_ID) REFERENCES REG_CONTENT (REG_CONTENT_ID))
/
CREATE INDEX REG_RESOURCE_PATH ON REG_RESOURCE (REG_PATH)
/
CREATE TABLE REG_DEPENDENCY (
                    REG_DEPENDENCY_ID INTEGER,
                    REG_PARENT_RID VARCHAR2 (50) NOT NULL,
                    REG_CHILD_RID VARCHAR2 (50) NOT NULL,
                    PRIMARY KEY (REG_DEPENDENCY_ID),
                    UNIQUE (REG_PARENT_RID, REG_CHILD_RID),
                    FOREIGN KEY (REG_PARENT_RID) REFERENCES REG_RESOURCE (REG_RID) ON DELETE CASCADE,
                    FOREIGN KEY (REG_CHILD_RID) REFERENCES REG_RESOURCE (REG_RID) ON DELETE CASCADE)
/
CREATE SEQUENCE REG_DEPENDENCY_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_DEPENDENCY_TRIGGER
                    BEFORE INSERT
                    ON REG_DEPENDENCY
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
                    IF :NEW.REG_PARENT_RID IS NOT NULL AND :NEW.REG_CHILD_RID IS NOT NULL THEN
                    SELECT REG_DEPENDENCY_SEQUENCE.nextval INTO :NEW.REG_DEPENDENCY_ID FROM dual;
		            END IF;
                    END;
/
CREATE TABLE REG_PROPERTY (
                    REG_PROPERTY_ID INTEGER,
                    REG_RID VARCHAR2 (50) NOT NULL,
                    REG_NAME VARCHAR2 (100) NOT NULL,
                    REG_PROPERTY_VALUE VARCHAR2 (500),
                    PRIMARY KEY (REG_PROPERTY_ID))
/
CREATE SEQUENCE REG_PROPERTY_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_PROPERTY_TRIGGER
                    BEFORE INSERT
                    ON REG_PROPERTY
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
                    IF :NEW.REG_RID IS NOT NULL AND :NEW.REG_NAME IS NOT NULL THEN
                    SELECT REG_PROPERTY_SEQUENCE.nextval INTO :NEW.REG_PROPERTY_ID FROM dual;
		            END IF;
                    END;
/
CREATE TABLE REG_ASSOCIATION (
                    REG_ASSOCIATION_ID INTEGER,
                    REG_SOURCEPATH VARCHAR2 (2000) NOT NULL,
                    REG_TARGETPATH VARCHAR2 (2000) NOT NULL,
                    REG_ASSOCIATION_TYPE VARCHAR2 (2000) NOT NULL,
                    PRIMARY KEY (REG_ASSOCIATION_ID))
/
CREATE SEQUENCE REG_ASSOCIATION_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_ASSOCIATION_TRIGGER
                    BEFORE INSERT
                    ON REG_ASSOCIATION
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
                    IF :NEW.REG_SOURCEPATH IS NOT NULL AND :NEW.REG_TARGETPATH IS NOT NULL AND :NEW.REG_ASSOCIATION_TYPE IS NOT NULL THEN
                    SELECT REG_ASSOCIATION_SEQUENCE.nextval INTO :NEW.REG_ASSOCIATION_ID FROM dual;
		            END IF;
                    END;
/
CREATE INDEX REG_ASSOCIATION_SOURCEPATH ON REG_ASSOCIATION (REG_SOURCEPATH)
/
CREATE INDEX REG_ASSOCIATION_TARGETPATH ON REG_ASSOCIATION (REG_TARGETPATH)
/
CREATE TABLE REG_TAG (
                    REG_TAG_ID INTEGER,
                    REG_TAG_NAME VARCHAR2 (500) NOT NULL,
                    REG_RID VARCHAR2 (50) NOT NULL,
                    REG_USER_ID VARCHAR2 (20) NOT NULL,
                    REG_TAGGED_TIME TIMESTAMP NOT NULL,
                    PRIMARY KEY (REG_TAG_ID),
                    UNIQUE (REG_TAG_NAME, REG_RID, REG_USER_ID))
/
CREATE SEQUENCE REG_TAG_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_TAG_TRIGGER
                    BEFORE INSERT
                    ON REG_TAG
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
                    IF :NEW.REG_TAG_NAME IS NOT NULL AND :NEW.REG_RID IS NOT NULL AND :NEW.REG_USER_ID IS NOT NULL AND :NEW.REG_TAGGED_TIME IS NOT NULL THEN
                    SELECT REG_TAG_SEQUENCE.nextval INTO :NEW.REG_TAG_ID FROM dual;
		            END IF;
                    END;
/
CREATE TABLE REG_COMMENT (
                    REG_COMMENT_ID INTEGER,
                    REG_RID VARCHAR2 (50) NOT NULL,
                    REG_USER_ID VARCHAR2 (20) NOT NULL,
                    REG_COMMENT_TEXT VARCHAR2 (500) NOT NULL,
                    REG_COMMENTED_TIME TIMESTAMP NOT NULL,
                    PRIMARY KEY (REG_COMMENT_ID))
/
CREATE SEQUENCE REG_COMMENT_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_COMMENT_TRIGGER
                    BEFORE INSERT
                    ON REG_COMMENT
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
		            IF :NEW.REG_RID IS NOT NULL AND :NEW.REG_USER_ID IS NOT NULL AND :NEW.REG_COMMENT_TEXT IS NOT NULL AND :NEW.REG_COMMENTED_TIME IS NOT NULL THEN
                    SELECT REG_COMMENT_SEQUENCE.nextval INTO :NEW.REG_COMMENT_ID FROM dual;
                    END IF;
                    END;
/
CREATE TABLE REG_RATING (
                    REG_RATING_ID INTEGER,
                    REG_RID VARCHAR2 (50) NOT NULL,
                    REG_USER_ID VARCHAR2 (20) NOT NULL,
                    REG_RATING INTEGER NOT NULL,
                    REG_RATED_TIME TIMESTAMP NOT NULL,
                    PRIMARY KEY (REG_RATING_ID))
/
CREATE SEQUENCE REG_RATING_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_RATING_TRIGGER
                    BEFORE INSERT
                    ON REG_RATING
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
                    IF :NEW.REG_RID IS NOT NULL AND :NEW.REG_USER_ID IS NOT NULL AND :NEW.REG_RATING IS NOT NULL AND :NEW.REG_RATED_TIME IS NOT NULL THEN
                    SELECT REG_RATING_SEQUENCE.nextval INTO :NEW.REG_RATING_ID FROM dual;
                    END IF;
                    END;
/
CREATE TABLE REG_LOG (
                    REG_LOG_ID INTEGER,
                    REG_PATH VARCHAR2 (2000),
                    REG_USER_ID VARCHAR2 (20) NOT NULL,
                    REG_LOGGED_TIME TIMESTAMP NOT NULL,
                    REG_ACTION INTEGER NOT NULL,
                    REG_ACTION_DATA VARCHAR2 (500),
                    PRIMARY KEY (REG_LOG_ID))
/
CREATE SEQUENCE REG_LOG_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_LOG_TRIGGER
                    BEFORE INSERT
                    ON REG_LOG
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
		            IF :NEW.REG_USER_ID IS NOT NULL AND :NEW.REG_LOGGED_TIME IS NOT NULL AND :NEW.REG_ACTION IS NOT NULL THEN
                    SELECT REG_LOG_SEQUENCE.nextval INTO :NEW.REG_LOG_ID FROM dual;
                    END IF;
		            END;
/
CREATE TABLE REG_CONTENT_VERSION (
                    REG_CONTENT_VERSION_ID VARCHAR2 (50),
                    REG_CONTENT_DATA BLOB,
                    PRIMARY KEY (REG_CONTENT_VERSION_ID))
/
CREATE TABLE REG_RESOURCE_VERSION (
                    REG_RESOURCE_VERSION_ID INTEGER,
                    REG_RID VARCHAR2 (50) NOT NULL,
                    REG_VERSION INTEGER NOT NULL,
                    REG_PATH VARCHAR2 (2000) NOT NULL,
                    REG_MEDIA_TYPE VARCHAR2 (500),
                    REG_COLLECTION INTEGER NOT NULL,
                    REG_CREATOR VARCHAR2 (500),
                    REG_CREATED_TIME TIMESTAMP,
                    REG_LAST_UPDATOR VARCHAR2 (500),
                    REG_LAST_UPDATED_TIME TIMESTAMP,
                    REG_DESCRIPTION VARCHAR2 (4000),
                    REG_CONTENT_ID VARCHAR2 (50),
                    REG_ASSOCIATED_SNAPSHOT_ID INTEGER NOT NULL,
                    FOREIGN KEY (REG_CONTENT_ID) REFERENCES REG_CONTENT_VERSION (REG_CONTENT_VERSION_ID),
                    PRIMARY KEY (REG_RESOURCE_VERSION_ID),
                    UNIQUE(REG_RID, REG_VERSION))
/
CREATE SEQUENCE REG_RESOURCE_VERSION_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_RESOURCE_VERSION_TRIGGER
                    BEFORE INSERT
                    ON REG_RESOURCE_VERSION
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
                    IF :NEW.REG_RID IS NOT NULL AND :NEW.REG_VERSION IS NOT NULL AND :NEW.REG_PATH IS NOT NULL AND :NEW.REG_COLLECTION IS NOT NULL AND :NEW.REG_ASSOCIATED_SNAPSHOT_ID IS NOT NULL THEN
                    SELECT REG_RESOURCE_VERSION_SEQUENCE.nextval INTO :NEW.REG_RESOURCE_VERSION_ID FROM dual;
                    END IF;
                    END;
/
CREATE TABLE REG_DEPENDENCY_VERSION (
                    REG_DEPENDENCY_VERSION_ID INTEGER,
                    REG_PARENT_RID VARCHAR2 (50) NOT NULL,
                    REG_PARENT_VERSION INTEGER NOT NULL,
                    REG_CHILD_RID VARCHAR2 (50) NOT NULL,
                    PRIMARY KEY (REG_DEPENDENCY_VERSION_ID),
                    UNIQUE (REG_PARENT_RID, REG_PARENT_VERSION, REG_CHILD_RID))
/
CREATE SEQUENCE REG_DEP_VERSION_SEQ START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_DEPENDENCY_VERSION_TRIGGER
                    BEFORE INSERT
                    ON REG_DEPENDENCY_VERSION
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
                    IF :NEW.REG_PARENT_RID IS NOT NULL AND :NEW.REG_PARENT_VERSION IS NOT NULL AND :NEW.REG_CHILD_RID IS NOT NULL THEN
                    SELECT REG_DEP_VERSION_SEQ.nextval INTO :NEW.REG_DEPENDENCY_VERSION_ID FROM dual;
		            END IF;
                    END;
/
CREATE TABLE REG_PROPERTY_VERSION (
                    REG_PROPERTY_VERSION_ID INTEGER,
                    REG_RID VARCHAR2 (50) NOT NULL,
                    REG_VERSION INTEGER NOT NULL,
                    REG_NAME VARCHAR2 (100) NOT NULL,
                    REG_PROPERTY_VALUE VARCHAR2 (500),
                    PRIMARY KEY (REG_PROPERTY_VERSION_ID),
                    FOREIGN KEY (REG_RID, REG_VERSION) REFERENCES REG_RESOURCE_VERSION (REG_RID, REG_VERSION))
/
CREATE SEQUENCE REG_PROPERTY_VERSION_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_PROPERTY_VERSION_TRIGGER
                    BEFORE INSERT
                    ON REG_PROPERTY_VERSION
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
                    IF :NEW.REG_RID IS NOT NULL AND :NEW.REG_VERSION IS NOT NULL AND :NEW.REG_NAME IS NOT NULL THEN
                    SELECT REG_PROPERTY_VERSION_SEQUENCE.nextval INTO :NEW.REG_PROPERTY_VERSION_ID FROM dual;
                    END IF;
                    END;
/
CREATE TABLE REG_SNAPSHOT (
                    REG_SNAPSHOT_ID INTEGER,
                    REG_ROOT_ID VARCHAR2 (50) NOT NULL,
                    PRIMARY KEY (REG_SNAPSHOT_ID),
                    UNIQUE (REG_SNAPSHOT_ID, REG_ROOT_ID))
/
CREATE SEQUENCE REG_SNAPSHOT_SEQUENCE START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_SNAPSHOT_TRIGGER
                    BEFORE INSERT
                    ON REG_SNAPSHOT
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
                    IF :NEW.REG_ROOT_ID IS NOT NULL THEN
                    SELECT REG_SNAPSHOT_SEQUENCE.nextval INTO :NEW.REG_SNAPSHOT_ID FROM dual;
		            END IF;
                    END;
/
CREATE INDEX REG_INDEX_SNAPSHOT_ROOT_ID ON REG_SNAPSHOT (REG_ROOT_ID)
/
CREATE TABLE REG_SNAPSHOT_RESOURCE_VERSION (
                    REG_SRV_ID INTEGER,
                    REG_SNAPSHOT_ID INTEGER NOT NULL,
                    REG_RID VARCHAR2 (50) NOT NULL,
                    REG_VERSION INTEGER NOT NULL,
                    PRIMARY KEY (REG_SRV_ID),
                    UNIQUE (REG_SNAPSHOT_ID, REG_RID, REG_VERSION))
/
CREATE SEQUENCE REG_SNAP_RESOURCE_VER_SEQ START WITH 1 INCREMENT BY 1 NOCACHE
/
CREATE OR REPLACE TRIGGER REG_SNAP_RESOURCE_VER_TRIGGER
                    BEFORE INSERT
                    ON REG_SNAPSHOT_RESOURCE_VERSION
                    REFERENCING NEW AS NEW
                    FOR EACH ROW
                    BEGIN
		            IF :NEW.REG_SNAPSHOT_ID IS NOT NULL AND :NEW.REG_RID IS NOT NULL AND :NEW.REG_VERSION IS NOT NULL THEN
                    SELECT REG_SNAP_RESOURCE_VER_SEQ.nextval INTO :NEW.REG_SRV_ID FROM dual;
	                END IF;
                    END;
/
commit;