/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.carbon.ui;

import org.apache.axis2.AxisFault;
import org.apache.axis2.client.Options;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.transport.http.HTTPConstants;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.eclipse.equinox.http.helper.BundleEntryHttpContext;
import org.osgi.framework.Bundle;
import org.wso2.carbon.CarbonConstants;
import org.wso2.carbon.registry.core.Registry;
import org.wso2.carbon.ui.authentication.AuthenticationAdminServiceStub;
import org.wso2.carbon.ui.authentication.GetSystemPermissionOfUser;
import org.wso2.carbon.ui.authentication.GetSystemPermissionOfUserResponse;
import org.wso2.carbon.ui.authentication.Login;
import org.wso2.carbon.ui.authentication.LoginResponse;
import org.wso2.carbon.utils.ServerConstants;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.util.ArrayList;

public class SecuredComponentEntryHttpContext extends BundleEntryHttpContext {

    private String bundlePath;

    protected Registry registry;

    protected Log log = LogFactory.getLog(this.getClass());
    private static final String AUTHENTICATION_ADMIN_SERVICE = "AuthenticationAdminService";

    public SecuredComponentEntryHttpContext(Bundle bundle) {
        super(bundle);
    }

    public SecuredComponentEntryHttpContext(Bundle bundle, String bundlePath) {
        super(bundle, bundlePath);
        this.bundlePath = bundlePath;
    }

    public SecuredComponentEntryHttpContext(Bundle bundle, String s, Registry registry) {
        super(bundle, s);
        this.registry = registry;
    }

    public URL getResource(String resourceName) {
        URL url = super.getResource(resourceName);
        if (url != null) {
            return url;
        } else {
            if (bundlePath != null) {
                resourceName = bundlePath + resourceName;
            }
            int lastSlash = resourceName.lastIndexOf('/');
            if (lastSlash == -1) {
                return null;
            }
            String file = resourceName.substring(lastSlash + 1);
            if (file.endsWith(".js") && isListed(file)) {
                try {
                    return new URL("carbon://generate/" + file);
                } catch (MalformedURLException e) {
                    return null;
                }
            }
            return null;

        }

    }

    /**
     * TODO: fix this from a configuration file
     *
     * @param file File
     * @return bool
     */
    private boolean isListed(String file) {
        return "global-params.js".equals(file);
    }

    protected boolean authenticate(String userName,
                                   String password,
                                   HttpServletRequest request) throws RemoteException {
        try {
            ServletContext servletContext = request.getSession().getServletContext();
            ConfigurationContext configContext = (ConfigurationContext) servletContext
                    .getAttribute(CarbonConstants.CONFIGURATION_CONTEXT);

            if (configContext == null) {
                log.error("Configuration context is null. Cannot retrieve configuration context from ServletContext");
            }

            String backendServerURL = request.getParameter("backendURL");
            if(backendServerURL == null){
                backendServerURL = CarbonUIUtil.getServerURL(servletContext, request.getSession());
            }
            String serviceURL = backendServerURL + AUTHENTICATION_ADMIN_SERVICE;
            AuthenticationAdminServiceStub stub =
                    new AuthenticationAdminServiceStub(configContext, serviceURL);
            Login loginRequest = new Login();
            loginRequest.setUsername(userName);
            loginRequest.setPassword(password);
            loginRequest.setRemoteAddress(request.getRemoteAddr());
            Options option = stub._getServiceClient().getOptions();
            option.setManageSession(true);
            LoginResponse loginResponse = stub.login(loginRequest);

            boolean isLogged = loginResponse.get_return();
            if (isLogged) {
                GetSystemPermissionOfUser request1 = new GetSystemPermissionOfUser();
                request1.setUsername(userName);
                GetSystemPermissionOfUserResponse permissions = stub.getSystemPermissionOfUser(request1);
                String[] permissionArray = permissions.get_return();
                HttpSession session = request.getSession();
                ArrayList<String> list = new ArrayList<String>();
                for (String permission : permissionArray) {
                    list.add(permission);
                }
                session.setAttribute(ServerConstants.USER_PERMISSIONS, list);
                String cookie = (String) stub._getServiceClient().getServiceContext().getProperty(
                        org.apache.axis2.transport.http.HTTPConstants.COOKIE_STRING);
                session.setAttribute(ServerConstants.ADMIN_SERVICE_COOKIE, cookie);
                session.setAttribute(CarbonConstants.SERVER_URL, backendServerURL);
            }
            return isLogged;
        } catch (AxisFault axisFault) {
            throw axisFault;
        } catch (RemoteException e) {
            throw e;
        } catch (Exception e){
            throw new AxisFault("Exception occured",e);
        }
    }

    protected void unauthenticate(HttpServletRequest request) {
        ServletContext servletContext = request.getSession().getServletContext();
        ConfigurationContext configContext = (ConfigurationContext) servletContext
                .getAttribute(CarbonConstants.CONFIGURATION_CONTEXT);

        String backendServerURL = CarbonUIUtil.getServerURL(servletContext, request.getSession());
        if(backendServerURL == null){
            return;
        }
        String serviceURL = backendServerURL + AUTHENTICATION_ADMIN_SERVICE;
        try {
            AuthenticationAdminServiceStub stub =
                    new AuthenticationAdminServiceStub(configContext, serviceURL);
            Options option = stub._getServiceClient().getOptions();
            String cookie = (String) request.getSession().getAttribute(ServerConstants.ADMIN_SERVICE_COOKIE);
            option.setProperty(HTTPConstants.COOKIE_STRING, cookie);
            option.setManageSession(true);
            stub.logout();
        } catch (RemoteException ignored) {
        } catch (Exception ignored){
        }
    }
}
