package org.wso2.carbon.user.core.dbcreators;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;

public class MSSQLDatabaseCreator implements DatabaseCreator{

    private DataSource dataSource = null;
    
    ////////////////////////////////////////////////////////////////////////////////////
    // User manager tables                                                            //
    ////////////////////////////////////////////////////////////////////////////////////

    public static String USER_TABLE = "CREATE TABLE UM_USERS ("
            + "ID INTEGER IDENTITY(1, 1)," + "USER_NAME VARCHAR(255) NOT NULL,"
            + "USER_PASSWORD VARCHAR(255) NOT NULL, " + "PRIMARY KEY (ID), " + "UNIQUE(USER_NAME))";

    public static String USER_ATTR_TABLE = "CREATE TABLE UM_USER_ATTRIBUTES ("
            + "ID INTEGER IDENTITY(1, 1)," + "ATTR_NAME VARCHAR(255) NOT NULL,"
            + "ATTR_VALUE VARCHAR(255), " + "USER_ID INTEGER, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static String ROLE_TABLE = "CREATE TABLE UM_ROLES ("
            + "ID INTEGER IDENTITY(1, 1), " + "ROLE_NAME VARCHAR(255) NOT NULL, "
            + "PRIMARY KEY (ID), " + "UNIQUE(ROLE_NAME))";

    public static String ROLE_ATTR_TABLE = "CREATE TABLE UM_ROLE_ATTRIBUTES ("
            + "ID INTEGER IDENTITY(1, 1), " + "ATTR_NAME VARCHAR(255) NOT NULL, "
            + "ATTR_VALUE VARCHAR(255), " + "ROLE_ID INTEGER, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static String PERMISSION_TABLE = "CREATE TABLE UM_PERMISSIONS ("
            + "ID INTEGER IDENTITY(1, 1), "
            + "RESOURCE_ID VARCHAR(255) NOT NULL, " + "ACTION VARCHAR(255) NOT NULL, "
            + "PRIMARY KEY (ID))";

    public static String PERMISSION_RESOUCE_ID_ACTION_INDEX =
            "CREATE INDEX INDEX_UM_PERMISSIONS_RESOURCE_ID_ACTION " +
                    "ON UM_PERMISSIONS (RESOURCE_ID, ACTION)";

    public static String ROLE_PERMISSION_TABLE = "CREATE TABLE UM_ROLE_PERMISSIONS ( "
            + "ID INTEGER IDENTITY(1, 1), " + "PERMISSION_ID INTEGER NOT NULL, "
            + "ROLE_ID INTEGER NOT NULL, " + "IS_ALLOWED SMALLINT NOT NULL, "
            + "UNIQUE (PERMISSION_ID, ROLE_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE  CASCADE, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static String USER_PERMISSION_TABLE = "CREATE TABLE UM_USER_PERMISSIONS ("
            + "ID INTEGER IDENTITY(1, 1), " + "PERMISSION_ID INTEGER NOT NULL, "
            + "USER_ID INTEGER NOT NULL, " + "IS_ALLOWED SMALLINT NOT NULL, "
            + "UNIQUE (PERMISSION_ID, USER_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static String USER_ROLES_TABLE = "CREATE TABLE UM_USER_ROLES ("
            + "ID INTEGER IDENTITY(1, 1), " + "ROLE_ID INTEGER NOT NULL, "
            + "USER_ID INTEGER NOT NULL, " + "UNIQUE (USER_ID, ROLE_ID), "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    
    // ////////////////////////////////////////////////////////////////
    // Hybrid sql statements
    // ////////////////////////////////////////////////////////////////
    
    public static String HYBRID_ROLE_TABLE = "CREATE TABLE HYBRID_ROLES ("
        + "ID INTEGER IDENTITY(1, 1), " + "ROLE_ID VARCHAR(255) NOT NULL, "
        + "PRIMARY KEY (ID), " + "UNIQUE(ROLE_ID))";
    
    public static final String HYBRID_USER_ROLE_TABLE = "CREATE TABLE HYBRID_USER_ROLES ("
        + "ID INTEGER IDENTITY(1, 1)," + "USER_ID VARCHAR(255),"
        + "ROLE_ID VARCHAR(255) NOT NULL," + "PRIMARY KEY (ID))";
    
    public static final String HYBRID_PERMISSION_TABLE = "CREATE TABLE HYBRID_PERMISSIONS ("
            + "ID INTEGER IDENTITY(1, 1)," + "RESOURCE_ID VARCHAR(255),"
            + "ACTION VARCHAR(255) NOT NULL," + "PRIMARY KEY (ID))";

    public static final String HYBRID_ROLE_PERMISSION_TABLE = "CREATE TABLE HYBRID_ROLE_PERMISSIONS ("
            + "ID INTEGER IDENTITY(1, 1)," + "PERMISSION_ID INTEGER NOT NULL,"
            + "ROLE_ID VARCHAR(255) NOT NULL," + "IS_ALLOWED SMALLINT NOT NULL,"
            + "UNIQUE (PERMISSION_ID, ROLE_ID),"
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES HYBRID_PERMISSIONS(ID) ON DELETE  CASCADE,"
            + "PRIMARY KEY (ID))";

    public static final String HYBRID_USER_PERMISSION_TABLE = "CREATE TABLE HYBRID_USER_PERMISSIONS ("
            + "ID INTEGER IDENTITY(1, 1)," + "PERMISSION_ID INTEGER NOT NULL,"
            + "USER_ID VARCHAR(255) NOT NULL," + "IS_ALLOWED SMALLINT NOT NULL,"
            + "UNIQUE (PERMISSION_ID, USER_ID),"
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES HYBRID_PERMISSIONS(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";
    
    public MSSQLDatabaseCreator(DataSource ds) {
        this.dataSource = ds;
    }
    
    public void createDefaultDatabaseTables() throws SQLException{
        Connection dbConnection = dataSource.getConnection();
        dbConnection.setAutoCommit(false);
        DatabaseMetaData dbmd = dbConnection.getMetaData();
        ResultSet rs = dbmd.getTables(null, null, "UM_USERS", null);
        if (rs.next() == false) {
            Statement s = dbConnection.createStatement();
            s.executeUpdate(USER_TABLE);
            s.executeUpdate(USER_ATTR_TABLE);
            s.executeUpdate(ROLE_TABLE);
            s.executeUpdate(ROLE_ATTR_TABLE);
            s.executeUpdate(PERMISSION_TABLE);
            s.executeUpdate(ROLE_PERMISSION_TABLE);
            s.executeUpdate(USER_PERMISSION_TABLE);
            s.executeUpdate(USER_ROLES_TABLE);
            s.executeUpdate(PERMISSION_RESOUCE_ID_ACTION_INDEX);
            dbConnection.commit();
        }
        dbConnection.close();
    }
    
    public void createHybridDatabaseTables() throws SQLException{
        Connection dbConnection = dataSource.getConnection();
        dbConnection.setAutoCommit(false);
        DatabaseMetaData dbmd = dbConnection.getMetaData();
        ResultSet rs = dbmd.getTables(null, null, "HYBRID_USER_ROLES", null);
        if (rs.next() == false) {
            Statement s = dbConnection.createStatement();
			s.executeUpdate(HYBRID_ROLE_TABLE);
            s.executeUpdate(HYBRID_USER_ROLE_TABLE);
            s.executeUpdate(HYBRID_PERMISSION_TABLE);
            s.executeUpdate(HYBRID_ROLE_PERMISSION_TABLE);
            s.executeUpdate(HYBRID_USER_PERMISSION_TABLE);
            dbConnection.commit();
        }
        dbConnection.close();
    }
}
