package org.wso2.carbon.user.core.dbcreators;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;

public class OracleDatabaseCreator implements DatabaseCreator {

    // //////////////////////////////////////////////////////////////////////////////////
    // User manager tables //
    // //////////////////////////////////////////////////////////////////////////////////

    public static String userTable = "CREATE TABLE UM_USERS (" + "ID INTEGER,"
            + "USER_NAME VARCHAR2(255) NOT NULL," + "USER_PASSWORD VARCHAR2(255) NOT NULL, "
            + "PRIMARY KEY (ID), " + "UNIQUE(USER_NAME))";

    public static final String userIDSequence = "CREATE SEQUENCE UM_USERS_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String userIDTrigger = "CREATE OR REPLACE TRIGGER UM_USERS_TRIGGER "
            + "BEFORE INSERT " + "ON UM_USERS " + "REFERENCING NEW AS NEW " + "FOR EACH ROW "
            + "BEGIN " + "SELECT UM_USERS_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";

    public static String userAttributeTable = "CREATE TABLE UM_USER_ATTRIBUTES (" + "ID INTEGER,"
            + "ATTR_NAME VARCHAR2(255) NOT NULL," + "ATTR_VALUE VARCHAR2(255), "
            + "USER_ID INTEGER, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static final String userAttributeIDSeq = "CREATE SEQUENCE UM_USER_ATTRIBUTES_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String userAttributeTrigger = "CREATE OR REPLACE TRIGGER UM_USER_ATTRIBUTES_TRIGGER "
            + "BEFORE INSERT "
            + "ON UM_USER_ATTRIBUTES "
            + "REFERENCING NEW AS NEW "
            + "FOR EACH ROW "
            + "BEGIN "
            + "SELECT UM_USER_ATTRIBUTES_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";

    public static String roleTable = "CREATE TABLE UM_ROLES (" + "ID INTEGER, "
            + "ROLE_NAME VARCHAR2(255) NOT NULL, " + "PRIMARY KEY (ID), " + "UNIQUE(ROLE_NAME))";

    public static final String roleIDSeq = "CREATE SEQUENCE UM_ROLES_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String roleIDTrigger = "CREATE OR REPLACE TRIGGER UM_ROLES_TRIGGER "
            + "BEFORE INSERT " + "ON UM_ROLES " + "REFERENCING NEW AS NEW " + "FOR EACH ROW "
            + "BEGIN " + "SELECT UM_ROLES_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";

    public static String roleAttributeTable = "CREATE TABLE UM_ROLE_ATTRIBUTES (" + "ID INTEGER, "
            + "ATTR_NAME VARCHAR2(255) NOT NULL, " + "ATTR_VALUE VARCHAR2(255), "
            + "ROLE_ID INTEGER, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static final String roleAttributesIDSeq = "CREATE SEQUENCE UM_ROLE_ATTRIBUTES_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String roleAttributesIDTrigger = "CREATE OR REPLACE TRIGGER UM_ROLE_ATTRIBUTES_TRIGGER "
            + "BEFORE INSERT "
            + "ON UM_ROLE_ATTRIBUTES "
            + "REFERENCING NEW AS NEW "
            + "FOR EACH ROW "
            + "BEGIN "
            + "SELECT UM_ROLE_ATTRIBUTES_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";

    public static String permissionTable = "CREATE TABLE UM_PERMISSIONS (" + "ID INTEGER, "
            + "RESOURCE_ID VARCHAR2(255) NOT NULL, " + "ACTION VARCHAR2(255) NOT NULL, "
            + "PRIMARY KEY (ID))";

    public static final String permissionIDSeq = "CREATE SEQUENCE UM_PERMISSIONS_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String permissionIDTrigger = "CREATE OR REPLACE TRIGGER UM_PERMISSIONS_TRIGGER "
            + "BEFORE INSERT "
            + "ON UM_PERMISSIONS "
            + "REFERENCING NEW AS NEW "
            + "FOR EACH ROW "
            + "BEGIN "
            + "SELECT UM_PERMISSIONS_SEQUENCE.nextval INTO :NEW.ID FROM dual; "
            + "END; ";

    public static String rolePermissionTable = "CREATE TABLE UM_ROLE_PERMISSIONS ( "
            + "ID INTEGER, " + "PERMISSION_ID INTEGER NOT NULL, " + "ROLE_ID INTEGER NOT NULL, "
            + "IS_ALLOWED SMALLINT NOT NULL, " + "UNIQUE (PERMISSION_ID, ROLE_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE  CASCADE, "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static final String rolePermissionIDSeq = "CREATE SEQUENCE UM_ROLE_PERMISSIONS_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String rolePermissionIDTrigger = "CREATE OR REPLACE TRIGGER UM_ROLE_PERMISSIONS_TRIGGER "
            + "BEFORE INSERT "
            + "ON UM_ROLE_PERMISSIONS "
            + "REFERENCING NEW AS NEW "
            + "FOR EACH ROW "
            + "BEGIN "
            + "SELECT UM_ROLE_PERMISSIONS_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";

    public static String userPermissionTable = "CREATE TABLE UM_USER_PERMISSIONS ("
            + "ID INTEGER, " + "PERMISSION_ID INTEGER NOT NULL, " + "USER_ID INTEGER NOT NULL, "
            + "IS_ALLOWED SMALLINT NOT NULL, " + "UNIQUE (PERMISSION_ID, USER_ID), "
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES UM_PERMISSIONS(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static final String userPermissionIDSeq = "CREATE SEQUENCE UM_USER_PERMISSIONS_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String userPermissionIDTrigger = "CREATE OR REPLACE TRIGGER UM_USER_PERMISSIONS_TRIGGER "
            + "BEFORE INSERT "
            + "ON UM_USER_PERMISSIONS "
            + "REFERENCING NEW AS NEW "
            + "FOR EACH ROW "
            + "BEGIN "
            + "SELECT UM_USER_PERMISSIONS_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";

    public static String userRolesTable = "CREATE TABLE UM_USER_ROLES (" + "ID INTEGER, "
            + "ROLE_ID INTEGER NOT NULL, " + "USER_ID INTEGER NOT NULL, "
            + "UNIQUE (USER_ID, ROLE_ID), "
            + "FOREIGN KEY (ROLE_ID) REFERENCES UM_ROLES(ID) ON DELETE CASCADE, "
            + "FOREIGN KEY (USER_ID) REFERENCES UM_USERS(ID) ON DELETE CASCADE, "
            + "PRIMARY KEY (ID))";

    public static final String userRolesIDSeq = "CREATE SEQUENCE UM_USER_ROLES_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String userRolesIDTrigger = "CREATE OR REPLACE TRIGGER UM_USER_ROLES_TRIGGER "
            + "BEFORE INSERT "
            + "ON UM_USER_ROLES "
            + "REFERENCING NEW AS NEW "
            + "FOR EACH ROW "
            + "BEGIN " + "SELECT UM_USER_ROLES_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";

    
    
    // ////////////////////////////////////////////////////////////////
    // Hybrid sql statements
    // ////////////////////////////////////////////////////////////////
    
    public static String HYBRID_ROLE_TABLE = "CREATE TABLE HYBRID_ROLES ("
        + "ID INTEGER, " + "ROLE_ID VARCHAR(255) NOT NULL, "
        + "PRIMARY KEY (ID), " + "UNIQUE(ROLE_ID))";
    
    public static final String HYBRID_ROLE_SEQ = "CREATE SEQUENCE HYBRID_ROLE_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String HYBRID_ROLE_TRIGGER = "CREATE OR REPLACE TRIGGER HYBRID_ROLE_TRIGGER "
            + "BEFORE INSERT " + "ON HYBRID_ROLES " + "REFERENCING NEW AS NEW " + "FOR EACH ROW "
            + "BEGIN " + "SELECT HYBRID_ROLE_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";
    
    public static final String HYBRID_USER_ROLE_TABLE = "CREATE TABLE HYBRID_USER_ROLES ("
        + "ID INTEGER," + "USER_ID VARCHAR(255),"
        + "ROLE_ID VARCHAR(255) NOT NULL," + "PRIMARY KEY (ID))";

    public static final String HYBRID_USER_ROLE_SEQ = "CREATE SEQUENCE HYBRID_USER_ROLE_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String HYBRID_USER_ROLE_TRIGGER = "CREATE OR REPLACE TRIGGER HYBRID_USER_ROLE_TRIGGER "
            + "BEFORE INSERT " + "ON HYBRID_USER_ROLES " + "REFERENCING NEW AS NEW " + "FOR EACH ROW "
            + "BEGIN " + "SELECT HYBRID_USER_ROLE_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";
    ///////
    public static final String HYBRID_PERMISSION_TABLE = "CREATE TABLE HYBRID_PERMISSIONS ("
            + "ID INTEGER," + "RESOURCE_ID VARCHAR(255),"
            + "ACTION VARCHAR(255) NOT NULL," + "PRIMARY KEY (ID))";

    public static final String HYBRID_PERMISSION_SEQ = "CREATE SEQUENCE HYBRID_PERMISSION_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String HYBRID_PERMISSION_TRIGGER = "CREATE OR REPLACE TRIGGER HYBRID_PERMISSION_TRIGGER "
            + "BEFORE INSERT " + "ON HYBRID_PERMISSIONS " + "REFERENCING NEW AS NEW " + "FOR EACH ROW "
            + "BEGIN " + "SELECT HYBRID_PERMISSION_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";
    ////////
    public static final String HYBRID_ROLE_PERMISSION_TABLE = "CREATE TABLE HYBRID_ROLE_PERMISSIONS ("
            + "ID INTEGER," + "PERMISSION_ID INTEGER NOT NULL,"
            + "ROLE_ID VARCHAR(255) NOT NULL," + "IS_ALLOWED SMALLINT NOT NULL,"
            + "UNIQUE (PERMISSION_ID, ROLE_ID),"
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES HYBRID_PERMISSIONS(ID) ON DELETE  CASCADE,"
            + "PRIMARY KEY (ID))";

    public static final String HYBRID_ROLE_PERMISSION_SEQ = "CREATE SEQUENCE HYBRID_ROLE_P_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String HYBRID_ROLE_PERMISSION_TRIGGER = "CREATE OR REPLACE TRIGGER HYBRID_ROLE_P_TRIGGER "
            + "BEFORE INSERT " + "ON HYBRID_ROLE_PERMISSIONS " + "REFERENCING NEW AS NEW " + "FOR EACH ROW "
            + "BEGIN " + "SELECT HYBRID_ROLE_P_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";
    /////////
    public static final String HYBRID_USER_PERMISSION_TABLE = "CREATE TABLE HYBRID_USER_PERMISSIONS ("
            + "ID INTEGER," + "PERMISSION_ID INTEGER NOT NULL,"
            + "USER_ID VARCHAR(255) NOT NULL," + "IS_ALLOWED SMALLINT NOT NULL,"
            + "UNIQUE (PERMISSION_ID, USER_ID),"
            + "FOREIGN KEY (PERMISSION_ID) REFERENCES HYBRID_PERMISSIONS(ID) ON DELETE CASCADE,"
            + "PRIMARY KEY (ID))";

    public static final String HYBRID_USER_PERMISSION_SEQ = "CREATE SEQUENCE HYBRID_USER_P_SEQUENCE START WITH 1 INCREMENT BY 1";

    public static final String HYBRID_USER_PERMISSION_TRIGGER = "CREATE OR REPLACE TRIGGER HYBRID_USER_P_TRIGGER "
            + "BEFORE INSERT " + "ON HYBRID_USER_PERMISSIONS " + "REFERENCING NEW AS NEW " + "FOR EACH ROW "
            + "BEGIN " + "SELECT HYBRID_USER_P_SEQUENCE.nextval INTO :NEW.ID FROM dual; " + "END; ";

    
    private DataSource dataSource = null;

    public OracleDatabaseCreator(DataSource ds) {
        this.dataSource = ds;
    }

    public void createDefaultDatabaseTables() throws SQLException {
        Connection dbConnection = dataSource.getConnection();
        DatabaseMetaData dbmd = dbConnection.getMetaData();
        ResultSet rs = dbmd.getTables(null, null, "UM_USERS", null);
        if (rs.next() == false) {
            Statement s = dbConnection.createStatement();
            s.executeUpdate(userTable);
            s.executeUpdate(userIDSequence);
            s.executeUpdate(userIDTrigger);
            
            s.executeUpdate(userAttributeTable);
            s.executeUpdate(userAttributeIDSeq);
            s.executeUpdate(userAttributeTrigger);
            
            s.executeUpdate(roleTable);
            s.executeUpdate(roleIDSeq);
            s.executeUpdate(roleIDTrigger);
            
            s.executeUpdate(roleAttributeTable);
            s.executeUpdate(roleAttributesIDSeq);
            s.executeUpdate(roleAttributesIDTrigger);
            
            s.executeUpdate(permissionTable);
            s.executeUpdate(permissionIDSeq);
            s.executeUpdate(permissionIDTrigger);
            
            s.executeUpdate(userPermissionTable);
            s.executeUpdate(userPermissionIDSeq);
            s.executeUpdate(userPermissionIDTrigger);
            
            s.executeUpdate(rolePermissionTable);
            s.executeUpdate(rolePermissionIDSeq);
            s.executeUpdate(rolePermissionIDTrigger);
            
            s.executeUpdate(userRolesTable);
            s.executeUpdate(userRolesIDSeq);
            s.executeUpdate(userRolesIDTrigger);
        }
    }

    public void createHybridDatabaseTables() throws SQLException {
        Connection dbConnection = dataSource.getConnection();
        DatabaseMetaData dbmd = dbConnection.getMetaData();
        ResultSet rs = dbmd.getTables(null, null, "HYBRID_USER_ROLES", null);
        if (rs.next() == false) {
            Statement s = dbConnection.createStatement();
            s.executeUpdate(HYBRID_ROLE_TABLE);
            s.executeUpdate(HYBRID_USER_ROLE_TABLE);
            s.executeUpdate(HYBRID_PERMISSION_TABLE);
            s.executeUpdate(HYBRID_ROLE_PERMISSION_TABLE);
            s.executeUpdate(HYBRID_USER_PERMISSION_TABLE);
            
            s.executeUpdate(HYBRID_ROLE_SEQ);
            s.executeUpdate(HYBRID_USER_ROLE_SEQ);
            s.executeUpdate(HYBRID_PERMISSION_SEQ);
            s.executeUpdate(HYBRID_ROLE_PERMISSION_SEQ);
            s.executeUpdate(HYBRID_USER_PERMISSION_SEQ);
            
            s.executeUpdate(HYBRID_ROLE_TRIGGER);
            s.executeUpdate(HYBRID_USER_ROLE_TRIGGER);
            s.executeUpdate(HYBRID_PERMISSION_TRIGGER);
            s.executeUpdate(HYBRID_ROLE_PERMISSION_TRIGGER);
            s.executeUpdate(HYBRID_USER_PERMISSION_TRIGGER);
            dbConnection.commit();
        }
        dbConnection.close();    
    }
}
