package org.wso2.carbon.user.core.hybrid;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.def.DefaultAccessControlAdmin;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;
import org.wso2.carbon.user.core.def.DefaultRealmConstants;
import org.wso2.carbon.user.core.def.SQLHelper;
import org.wso2.carbon.user.core.i18n.Messages;

public class HybridAccessControlAdmin extends HybridAuthorizer implements AccessControlAdmin {
    private static Log log = LogFactory.getLog(DefaultAccessControlAdmin.class);

    DefaultAccessControlAdmin defacControlAdmin = null;

    public HybridAccessControlAdmin(DataSource dataSource, String algo, HybridRealmConfig config) {
        super(dataSource, algo, config);
        defacControlAdmin = new DefaultAccessControlAdmin(dataSource, algo, sqlHelper);
    }

    public HybridAccessControlAdmin(DataSource dataSource, HybridRealmConfig config) {
        super(dataSource, DefaultRealmConfig.PERMISSION_BLOCK_FIRST, config);
        defacControlAdmin = new DefaultAccessControlAdmin(dataSource,
                DefaultRealmConfig.PERMISSION_BLOCK_FIRST, sqlHelper);
    }

    public void authorizeRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        this.addAllowDenyRolePermission(roleName, resourceId, action, (short)1);

    }

    public void authorizeUser(String userName, String resourceId, String action)
            throws UserStoreException {
        this.addAllowDenyUserPermission(userName, resourceId, action, (short)1);
    }

    public void denyRole(String roleName, String resourceId, String action)
            throws UserStoreException {
        this.addAllowDenyRolePermission(roleName, resourceId, action, (short)0);

    }

    public void denyUser(String userName, String resourceId, String action)
            throws UserStoreException {
        this.addAllowDenyUserPermission(userName, resourceId, action, (short)0);
    }

    public void clearResourceAuthorizations(String resourceId) throws UserStoreException {
        defacControlAdmin.clearResourceAuthorizations(resourceId);
    }

    public void copyAuthorizations(String fromResourceId, String toResourceId)
            throws UserStoreException {
	Connection dbConnection = null;
	try {
	    dbConnection = dataSource.getConnection();
	    if (dbConnection == null) {
		throw new UserStoreException(Messages.getMessage("null_connection"));
	    }
	    dbConnection.setAutoCommit(false);
	    PreparedStatement getResourcePermissionCountStmt = dbConnection
		    .prepareStatement(sqlHelper
			    .getSQL(DefaultRealmConstants.GET_RESOURCE_PERMISSION_COUNT));
	    getResourcePermissionCountStmt.setString(1, toResourceId);
	    ResultSet rs = getResourcePermissionCountStmt.executeQuery();
	    if (rs.next()) {
		int permissionCount = rs.getInt(DefaultRealmConstants.COLUMN_NAME_PERMISSION_COUNT);
		if (permissionCount > 0) {
		    throw new UserStoreException(Messages
			    .getMessage("copyAuthorizationFailedPermissionExist"));
		}
	    }

	    String addRoleSQL = sqlHelper.getSQL(DefaultRealmConstants.ADD_ROLE_PERMISSION);
	    int index = addRoleSQL.lastIndexOf("(");
	    addRoleSQL = addRoleSQL.substring(0, index - 1);
	    String addUserSQL = sqlHelper.getSQL(DefaultRealmConstants.ADD_USER_PERMISSION);
	    index = addUserSQL.lastIndexOf("(");
	    addUserSQL = addUserSQL.substring(0, index - 1);

	    List<String> rplist = new ArrayList<String>();
	    List<String> uplist = new ArrayList<String>();

	    PreparedStatement getPermissionStmt = dbConnection.prepareStatement(sqlHelper
		    .getSQL(DefaultRealmConstants.GET_PERMISSION));

	    PreparedStatement addPermissionStmt = dbConnection.prepareStatement(sqlHelper
		    .getSQL(DefaultRealmConstants.ADD_PERMISSION));

	    PreparedStatement getResourcePermissionStmt = dbConnection.prepareStatement(sqlHelper
		    .getSQL(DefaultRealmConstants.GET_RESOURCE_PERMISSION));
	    getResourcePermissionStmt.setString(1, fromResourceId);
	    rs = getResourcePermissionStmt.executeQuery();
	    Map<String, String> permission = new HashMap<String, String>();
	    String permissionString = "";
	    while (rs.next()) {
		String action = rs.getString(DefaultRealmConstants.COLUMN_NAME_ACTION);
		String oldPermissionId = rs.getString(DefaultRealmConstants.COLUMN_NAME_ID);
		permissionString = permissionString + oldPermissionId + ",";

		addPermissionStmt.setString(1, toResourceId);
		addPermissionStmt.setString(2, action);
		addPermissionStmt.executeUpdate();

		getPermissionStmt.setString(1, toResourceId);
		getPermissionStmt.setString(2, action);

		ResultSet rs2 = getPermissionStmt.executeQuery();
		String idPermission = null;
		if (rs2.next()) {
		    idPermission = rs2.getString(DefaultRealmConstants.COLUMN_NAME_ID);
		}

		permission.put(oldPermissionId, idPermission);
	    }
	    getResourcePermissionStmt.close();
	    addPermissionStmt.close();
	    getPermissionStmt.close();
	    
	    if(permissionString.length()==0){
		return;
	    }
	    
	    permissionString = permissionString.substring(0, permissionString.length() - 1);
	    
	    //add role permission
	    String rpStmt = sqlHelper.getSQL(DefaultRealmConstants.GET_ROLE_PERMISSIONS);
	    rpStmt = rpStmt + "(" + permissionString + ")";
	    Statement rstmt = dbConnection.createStatement();
	    ResultSet rsr = rstmt.executeQuery(rpStmt);

	    while (rsr.next()) {
		String roleId = rsr.getString(DefaultRealmConstants.COLUMN_NAME_ROLE_ID);
		short allow = rsr.getShort(DefaultRealmConstants.COLUMN_NAME_IS_ALLOWED);
		String oldId = rsr.getString(DefaultRealmConstants.COLUMN_NAME_PERMISSION_ID);
		String newId = permission.get(oldId);
		String sql = addRoleSQL + " (" + newId + " , " + allow + " , '" + roleId + "')";
		rplist.add(sql);
	    }

	    rstmt.close();

	    //add user permission
	    String upStmt = sqlHelper.getSQL(DefaultRealmConstants.GET_USER_PERMISSIONS);
	    upStmt = upStmt + "(" + permissionString + ")";
	    Statement ustmt = dbConnection.createStatement();
	    ResultSet rsu = ustmt.executeQuery(upStmt);

	    while (rsu.next()) {
		String userId = rsu.getString(DefaultRealmConstants.COLUMN_NAME_USER_ID);
		short allow = rsu.getShort(DefaultRealmConstants.COLUMN_NAME_IS_ALLOWED);
		String oldId = rsu.getString(DefaultRealmConstants.COLUMN_NAME_PERMISSION_ID);
		String newId = permission.get(oldId);
		String sql = addUserSQL + " (" + newId + " , " + allow + " , '" + userId + "')";
		uplist.add(sql);
	    }
	    
	    ustmt.close();

	    Statement s = dbConnection.createStatement();
	    s.clearBatch();
	    String[] array1 = rplist.toArray(new String[rplist.size()]);
	    String[] array2 = uplist.toArray(new String[uplist.size()]);

	    for (String sqlSt : array1) {
		s.addBatch(sqlSt);
	    }
	    for (String sqlSt : array2) {
		s.addBatch(sqlSt);
	    }
	    s.executeBatch();
	    s.close();

	   
	    dbConnection.commit();

	} catch (SQLException e) {
	    log.debug(e);
	    throw new UserStoreException(Messages.getMessage("errorCopyingAuthorizations"), e);
	} finally {
	    try {
		if (dbConnection != null) {
		    dbConnection.close();
		}
	    } catch (SQLException e) {
		throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
	    }
	}
    }

    public void clearUserAuthorization(String userName, String resourceId, String action)
            throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getPermission = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.GET_PERMISSION));
            getPermission.setString(1, resourceId);
            getPermission.setString(2, action);

            ResultSet rs = getPermission.executeQuery();
            int pid;
            if (rs.next()) {
                pid = rs.getInt(sqlHelper.getColumnName(DefaultRealmConstants.COLUMN_ID_ID));
            } else {
                throw new UserStoreException(Messages.getMessage("nullData"));
            }

            PreparedStatement clearUPStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_USER_PERMISSION));
            clearUPStmt.setString(1, userName);
            clearUPStmt.setInt(2, pid);
            clearUPStmt.executeUpdate();
            dbConnection.commit();
            getPermission.close();
            clearUPStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            e.printStackTrace();
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }

    public void clearRoleAuthorization(String roleName, String resourceId, String action)
            throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            PreparedStatement getPermission = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.GET_PERMISSION));
            getPermission.setString(1, resourceId);
            getPermission.setString(2, action);

            ResultSet rs = getPermission.executeQuery();
            String pid;
            if (rs.next()) {
                pid = rs.getString(DefaultRealmConstants.COLUMN_NAME_ID);
            } else {
                throw new UserStoreException(Messages.getMessage("nullData"));
            }

            PreparedStatement deleteRolesStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_ROLE_PERMISSION));
            deleteRolesStmt.setString(1, roleName);
            deleteRolesStmt.setString(2, pid);
            deleteRolesStmt.executeUpdate();

            dbConnection.commit();
            getPermission.close();
            deleteRolesStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }

    }

    protected void addAllowDenyRolePermission(String roleName, String resourceId, String action,
            short allow) throws UserStoreException {
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

            int permissionId = defacControlAdmin.getOrAddPermissionId(dbConnection, resourceId, action);

            PreparedStatement deleteRolesStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_ROLE_PERMISSION));
            deleteRolesStmt.setString(1, roleName);
            deleteRolesStmt.setInt(2, permissionId);
            deleteRolesStmt.executeUpdate();

            PreparedStatement addRolePermissionStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ADD_ROLE_PERMISSION));
            addRolePermissionStmt.setInt(1, permissionId);
            addRolePermissionStmt.setShort(2, allow);
            addRolePermissionStmt.setString(3, roleName);
            addRolePermissionStmt.executeUpdate();
            dbConnection.commit();
            addRolePermissionStmt.close();
            deleteRolesStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }

    protected void addAllowDenyUserPermission(String userName, String resourceId, String action,
            short allow) throws UserStoreException {
        Connection dbConnection = null;

        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);

            int permissionId = defacControlAdmin.getOrAddPermissionId(dbConnection, resourceId, action);

            PreparedStatement clearUPStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.DELETE_USER_PERMISSION));
            clearUPStmt.setString(1, userName);
            clearUPStmt.setInt(2, permissionId);
            clearUPStmt.executeUpdate();

            PreparedStatement addUserPermissionStmt = dbConnection.prepareStatement(sqlHelper
                    .getSQL(DefaultRealmConstants.ADD_USER_PERMISSION));
            addUserPermissionStmt.setInt(1, permissionId);
            addUserPermissionStmt.setShort(2, allow);
            addUserPermissionStmt.setString(3, userName);
            addUserPermissionStmt.executeUpdate();
            dbConnection.commit();
            addUserPermissionStmt.close();
            clearUPStmt.close();
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }

    }

}
