package org.wso2.carbon.user.core.hybrid;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;

import javax.sql.DataSource;

import org.apache.commons.dbcp.BasicDataSource;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.Authenticator;
import org.wso2.carbon.user.core.Authorizer;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.def.DefaultAccessControlAdmin;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;
import org.wso2.carbon.user.core.def.DefaultRealmConstants;
import org.wso2.carbon.user.core.i18n.Messages;


public class HybridRealm implements UserRealm {

    private static Log log = LogFactory.getLog(HybridRealm.class);
    
    private Authenticator authenticator = null;

    private Authorizer authorizer = null;
    private AccessControlAdmin aclAdmin = null;

    private UserStoreAdmin usAdmin = null;
    private UserStoreReader usReader = null;

    private DataSource dataSource = null;
    private HybridRealmConfig config = null;
    private HybridInternalRoleAdmin internalRoleAdmin = null;
  
    
    public void init(Object configBean) throws UserStoreException {

        if (!(configBean instanceof HybridRealmConfig)) {
            return;
        }
        HybridRealmConfig config = (HybridRealmConfig) configBean;

        BasicDataSource basicDataSource = (BasicDataSource)config.getDataSource();
        if (basicDataSource == null) {
            basicDataSource = new BasicDataSource();
            basicDataSource.setUrl(config.getConnectionURL());
            basicDataSource.setDriverClassName(config.getDriverName());
            basicDataSource.setUsername(config.getConnectionUserName());
            basicDataSource.setPassword(config.getConnectionPassword());
        }
        this.config = config;
        dataSource = (DataSource) basicDataSource;

        if(config.getAuthenticator() == null)
            throw new UserStoreException("authenticatorMustExist");
        
        authenticator = config.getAuthenticator();
        
        usAdmin = config.getUserStoreAdmin();
        UserStoreReader reader = config.getUserStoreReader();
        usReader = new HybridUserStoreReader(dataSource, reader);
        
        authorizer = new HybridAuthorizer(dataSource,
                DefaultRealmConfig.PERMISSION_BLOCK_FIRST, config);
        aclAdmin = new HybridAccessControlAdmin(dataSource, config);
        
        internalRoleAdmin = new HybridInternalRoleAdmin(dataSource);
        
    }

    public void cleanUp() throws UserStoreException{
        Connection dbConnection = null;
        try {
            dbConnection = dataSource.getConnection();
            if (dbConnection == null) {
                throw new UserStoreException(Messages.getMessage("null_connection"));
            }
            dbConnection.setAutoCommit(false);
            Statement stmt = dbConnection.createStatement();
            stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_ROLE_TABLE);
            stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_USER_ROLE_TABLE);
            stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_PERMISSION_TABLE);
            stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_ROLE_PERMISSION_TABLE);
            stmt.executeUpdate(HybridRealmConstants.DELETE_HYBRID_USER_PERMISSION_TABLE); 
                     
            dbConnection.commit();
            
        } catch (SQLException e) {
            log.debug(e);
            throw new UserStoreException(Messages.getMessage("errorModifyingUserStore"), e);
        } finally {
            try {
                if (dbConnection != null) {
                    dbConnection.close();
                }
            } catch (SQLException e) {
                throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
            }
        }
    }
    
    public AccessControlAdmin getAccessControlAdmin() throws UserStoreException {
        return aclAdmin;
    }

    public Authenticator getAuthenticator() throws UserStoreException {
        return authenticator;
    }

    public Authorizer getAuthorizer() throws UserStoreException {
        return authorizer;
    }

    public UserStoreAdmin getUserStoreAdmin() throws UserStoreException {
        return this.usAdmin;
    }

    public UserStoreReader getUserStoreReader() throws UserStoreException {
        return this.usReader;
    }

    public HybridInternalRoleAdmin getInternalRoleAdmin() throws UserStoreException {
        return internalRoleAdmin;
    }
    
    public Object getRealmConfiguration() throws UserStoreException {
        HybridRealmConfig retConfig = null;
        if (config == null) {
            retConfig = new HybridRealmConfig();
        }
        return retConfig;
    }

    
    public DataSource getDataSource() {
        return dataSource;
    }

    @Override
    protected void finalize() throws Throwable {
	dataSource = null;
	super.finalize();
    }
    
    
}
