package org.wso2.carbon.user.core.hybrid.jdbc;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.def.DefaultRealmConstants;
import org.wso2.carbon.user.core.i18n.Messages;

public class JDBCUserStoreReader implements UserStoreReader{

    private static Log log = LogFactory.getLog(JDBCUserStoreReader.class);
    protected JDBCRealmConfig config = null;
    
    public JDBCUserStoreReader(JDBCRealmConfig config){
        this.config = config;
    }
    public String[] getAllRoleNames() throws UserStoreException {
        String[] names = new String[0];
        String sqlstmt = config.getRoleListSQL();
        if(sqlstmt != null){
            names = this.retriveNames(sqlstmt, null);
        }
        return names;
    }

    public String[] getAllUserNames() throws UserStoreException {
        String[] names = new String[0];
        String sqlstmt = config.getUserListSQL();
        if(sqlstmt != null){
            names = this.retriveNames(sqlstmt, null);
        }
        return names;
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
        String[] roleNames = this.getAllRoleNames();
        for(String name : roleNames){
            if(name.equals(roleName)){
                return true;
            }
        }
        return false;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
        String sqlstmt = config.getIsUserExistingSQL();
        boolean isExisting = false;
        Connection dbConnection = null;
        try {
            dbConnection = config.getDataSource().getConnection();
            dbConnection.setAutoCommit(false);
            PreparedStatement prepStmt = dbConnection.prepareStatement(sqlstmt);
            prepStmt.setString(1, userName);
            ResultSet rs = prepStmt.executeQuery();
            while (rs.next()) {
        	isExisting = true;
            }
            
        } catch (SQLException e) {
            log.error(e);
            // do not throw the exception. system should function properly
        }  finally {
            try {       
                if(dbConnection!=null){
                    dbConnection.close();
                }
            } catch (SQLException e) {
        	log.error(e);
             
            }
        }
        return isExisting;
    }


    public String[] getUserRoles(String userName) throws UserStoreException {
        String[] names = new String[0];
        String sqlstmt = config.getUserRoleSQL();
        if(sqlstmt != null){
            names = this.retriveNames(sqlstmt, userName);
        }
        return names;
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
        String sqlstmt = config.getUsersInRoleSQL();
        return this.retriveNames(sqlstmt, roleName);
    }

////// ***************************** Not going to implement ************************    
    
    public Map<String, String> getRoleProperties(String roleName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public List<String> getUserProfileNames(String userName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Map<String, String> getUserProperties(String userName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public Map<String, String> getUserProperties(String userName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }

    public String[] getUserPropertyNames() throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }


    public boolean isExistingUserProfile(String userName, String profileName)
            throws UserStoreException {
        // TODO Auto-generated method stub
        return false;
    }
    
    public String getDefaultUserProfileName(String userName) throws UserStoreException {
        // TODO Auto-generated method stub
        return null;
    }
    
    public String[] listUsers(String filter, int maxItemLimit) throws UserStoreException{
	String[] users = new String[0];
	Connection dbConnection = null;
	try {
	    String sqlStmt = config.getUserFilterSQL();
	    if (filter != null && filter.trim().length() != 0) {
		filter = filter.trim();
		filter = filter.replace("*", "%");
		filter = filter.replace("?", "_");
		filter = filter;
	    }else{
		filter = "%";
	    }

	    dbConnection = config.getDataSource().getConnection();
	    
	    if (dbConnection == null) {
		throw new UserStoreException(Messages.getMessage("null_connection"));
	    }
	    dbConnection.setAutoCommit(false);
	    dbConnection.setTransactionIsolation(Connection.TRANSACTION_READ_COMMITTED);
	    
	    PreparedStatement prepStmt = dbConnection.prepareStatement(sqlStmt);
	    prepStmt.setString(1, filter);
	    ResultSet rs = prepStmt.executeQuery();

	    List lst = new LinkedList();
	    
	    int i = 0;
	    while (rs.next()) {
		if (i < maxItemLimit) {
		    lst.add(rs.getString(1));
		}else{
		    break;
		}
		i++;
	    }
	    rs.close();

	    if (lst.size() > 0) {
		users = (String[]) lst.toArray(new String[lst.size()]);
	    }

	} catch (SQLException e) {
	    log.debug(e);
	    throw new UserStoreException(Messages.getMessage("errorReadingFromUserStore"), e);
	} finally {
	    try {
		if (dbConnection != null) {
		    dbConnection.close();
		}
	    } catch (SQLException e) {
		throw new UserStoreException(Messages.getMessage("errorClosingConnection"), e);
	    }
	}
	return users;
    }
    
    
    private String[] retriveNames(String sqlstmt, String param) throws UserStoreException{
        String[] names = new String[0];
        
        if(sqlstmt == null){
            return names;
        }
        
        Connection dbConnection = null;
        try {
            dbConnection = config.getDataSource().getConnection();
            dbConnection.setAutoCommit(false);
            PreparedStatement prepStmt = dbConnection.prepareStatement(sqlstmt);
            if(param != null){
                prepStmt.setString(1, param);
            }
            ResultSet rs = prepStmt.executeQuery();
            List<String> lst = new ArrayList<String>();
            while (rs.next()) {
                String name = rs.getString(1);
                lst.add(name);
            }
            if (lst.size() > 0) {
                names = (String[]) lst.toArray(new String[lst.size()]);
            }
        } catch (SQLException e) {
            log.error(e);
            // do not throw the exception. system should function properly
        }  finally {
            try {       
                if(dbConnection!=null){
                    dbConnection.close();
                }
            } catch (SQLException e) {
        	log.error(e);
             
            }
        }
        return names;
        
    }


}
