package org.wso2.carbon.user.core.hybrid.ldap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.DirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.sql.DataSource;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.carbon.user.core.UserStoreException;
import org.wso2.carbon.user.core.UserStoreReader;
import org.wso2.carbon.user.core.i18n.Messages;

public class LDAPUserStoreReader implements UserStoreReader {

    protected DataSource dataSource = null;

    private static Log log = LogFactory.getLog(LDAPUserStoreReader.class);

    private LDAPRealmConfig config = null;

    public LDAPUserStoreReader(LDAPRealmConfig config) {
	this.config = config;
    }

    public String[] getAllUserNames() throws UserStoreException {
	String[] userNames = new String[0];

	try {
	    DirContext context = config.getContext();
	    NamingEnumeration answer = context.list(config.getUserContextName());
	    List lst = new ArrayList();
	    while (answer.hasMore()) {
		Object obj = answer.nextElement();
		lst.add(obj.toString());
	    }
	    String[] wholeNames = (String[]) lst.toArray(new String[lst.size()]);
	    String[] names = new String[lst.size()];
	    for (int i = 0; i < wholeNames.length; i++) {
		String[] temp1 = wholeNames[i].split(":");
		if (temp1.length > 0) {
		    String[] temp2 = temp1[0].split("=");
		    if (temp2.length > 1) {
			names[i] = temp2[1].trim();
		    } else {
			throw new UserStoreException(Messages
				.getMessage("unknownUserNameFormatInLDAP"));
		    }
		} else {
		    throw new UserStoreException(Messages.getMessage("unknownUserNameFormatInLDAP"));
		}
	    }
	    userNames = names;
	} catch (NamingException e) {
	    //do not throw the exception. system should function properly
	}
	return userNames;
    }

    public boolean isExistingUser(String userName) throws UserStoreException {
	boolean bFound = false;
	SearchControls searchCtls = new SearchControls();
	searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
	

	String searchFilter = "(&(objectClass=person)(uid=" + userName + "))"; //RFC 1274
	String searchBase = config.getUserContextName();

	String returnedAtts[] = { "uid" };
	searchCtls.setReturningAttributes(returnedAtts);
	try {
	    NamingEnumeration answer = config.getContext().search(searchBase, searchFilter,
		    searchCtls);
	    while (answer.hasMoreElements()) {
		SearchResult sr = (SearchResult) answer.next();
		bFound = true;
	    }
	   
	} catch (NamingException e) {
	    log.error("Problem getting usernames: " + e);
	    // do not throw the exception. system should function properly
	}

	return bFound;
    }

    public String[] getUserRoles(String userName) throws UserStoreException {
	return new String[0];
    }

    public String[] getAllRoleNames() throws UserStoreException {
	return new String[0];
    }

    // ////***************************** Not going to implement
    // ************************
    public Map getUserProperties(String userName) throws UserStoreException {
	return null;
    }

    public String[] getUserPropertyNames() throws UserStoreException {
	return null;
    }

    public String[] getUserNamesWithPropertyValue(String propertyName, String propetyValue)
	    throws UserStoreException {
	return null;
    }

    // default style

    public Map getRoleProperties(String roleName) throws UserStoreException {
	return new HashMap();
    }

    public String[] getUsersInRole(String roleName) throws UserStoreException {
	return new String[0];
    }

    public boolean isExistingRole(String roleName) throws UserStoreException {
	return false;
    }

    public Map<String, String> getUserProperties(String userName, String profileName)
	    throws UserStoreException {
	// TODO 1.0
	return null;
    }

    public List<String> getUserProfileNames(String userName) throws UserStoreException {
	// TODO 1.0
	return null;
    }

    public String getDefaultUserProfileName(String userName) throws UserStoreException {
	// TODO 1.0
	return null;
    }

    public boolean isExistingUserProfile(String userName, String profileName)
	    throws UserStoreException {
	// TODO 1.0
	return false;
    }

    public String[] listUsers(String filter, int maxItemLimit) throws UserStoreException {
	String[] userNames = new String[0];
	SearchControls searchCtls = new SearchControls();
	searchCtls.setSearchScope(SearchControls.SUBTREE_SCOPE);
	searchCtls.setCountLimit(maxItemLimit);

	if (filter.contains("?")) {
	    throw new UserStoreException("invalid character for act dir serch");
	}

	String searchFilter = "(&(objectClass=person)(uid=" + filter + "))"; //RFC 1274
	String searchBase = config.getUserContextName();

	String returnedAtts[] = { "uid" };
	searchCtls.setReturningAttributes(returnedAtts);
	try {
	    NamingEnumeration answer = config.getContext().search(searchBase, searchFilter,
		    searchCtls);
	    List<String> list = new ArrayList<String>();
	    int i = 0;
	    while (answer.hasMoreElements() && i < maxItemLimit) {
		SearchResult sr = (SearchResult) answer.next();
		String name = (String) sr.getAttributes().get("uid").get();
		list.add(name);
		i++;
	    }
	    userNames = list.toArray(new String[list.size()]);
	} catch (NamingException e) {
	    log.error("Problem getting usernames: " + e);
	    // do not throw the exception. system should function properly
	}
	return userNames;
    }
}
