package org.wso2.carbon.user.core.internal;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Dictionary;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;

import javax.sql.DataSource;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLStreamException;

import net.sf.ehcache.Cache;
import net.sf.ehcache.CacheManager;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.impl.builder.StAXOMBuilder;
import org.apache.commons.dbcp.BasicDataSource;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.util.tracker.ServiceTracker;
import org.wso2.carbon.user.core.AccessControlAdmin;
import org.wso2.carbon.user.core.SystemPermissions;
import org.wso2.carbon.user.core.UserCoreConstants;
import org.wso2.carbon.user.core.UserRealm;
import org.wso2.carbon.user.core.UserStoreAdmin;
import org.wso2.carbon.user.core.dbcreators.DBCreatorFactory;
import org.wso2.carbon.user.core.dbcreators.DatabaseCreator;
import org.wso2.carbon.user.core.def.DefaultRealm;
import org.wso2.carbon.user.core.def.DefaultRealmConfig;

	

public class Activator implements BundleActivator {
    private ServiceRegistration defaultRealmReference = null;
    private ServiceRegistration hybridRealmReference = null;
    
    public void start(BundleContext bc) throws Exception {
        try {
            
            OMElement parentElement = getConfigurationElement();
            
            OMElement dbelement = parentElement.getFirstChildWithName(new QName("Database"));
            String url = dbelement.getFirstChildWithName(new QName(UserCoreConstants.UserMgtConfig.URL)).getText();
            String password = dbelement.getFirstChildWithName(new QName(UserCoreConstants.UserMgtConfig.PASSWORD)).getText();
            String userName = dbelement.getFirstChildWithName(new QName(UserCoreConstants.UserMgtConfig.USERNAME)).getText();
            String driverName = dbelement.getFirstChildWithName(new QName(UserCoreConstants.UserMgtConfig.DRIVER)).getText();
            String dialect = dbelement.getFirstChildWithName(new QName(UserCoreConstants.UserMgtConfig.DIALECT)).getText();
            
            BasicDataSource dataSource = new BasicDataSource();
            dataSource.setUrl(url);
            dataSource.setDriverClassName(driverName);
            dataSource.setUsername(userName);
            dataSource.setPassword(password);

            initializeDatabase(dataSource, dialect);

            //UserMgtConfigurationContext.init(bc, url, userName, password, driverName);

            UserRealm userRealm = exposeDefaultRealmAsService(bc, url, driverName, userName, password);
            this.addInitialData(parentElement, userRealm);
            
        } catch (RuntimeException e) {
            e.printStackTrace();
        }
        
    }

    public void stop(BundleContext bundleContext) throws Exception{
	ServiceReference[] serviceRefs = bundleContext.getServiceReferences(UserRealm.class.getName(), "(RealmGenre=Default)");
	if(serviceRefs != null && serviceRefs.length>0){
	    DefaultRealm realm = (DefaultRealm)bundleContext.getService(serviceRefs[0]);
	    DataSource ds = realm.getDataSource();
	    ds=null;
	}
    }
  
    protected OMElement getConfigurationElement(){
        try {
            String carbonHome = System.getProperty("carbon.home");
            if(carbonHome == null){
                carbonHome = System.getenv("CARBON_HOME");
            }
            
            InputStream inStream = null;
            if (carbonHome != null) {
                File userMgtXml = new File(carbonHome + File.separator + "conf", "user-mgt.xml");
                if (!userMgtXml.exists()) {
                    String msg =
                            "Instance of a WSO2 User Manager has not been created. user-mgt.xml is not found.";
                    throw new FileNotFoundException(msg);
                }
                inStream  = new FileInputStream(userMgtXml);
            } else {
                inStream = this.getClass().getClassLoader().getResourceAsStream("conf/user-mgt.xml");
                if(inStream == null){
                    String msg =
                        "Instance of a WSO2 User Manager has not been created. user-mgt.xml is not found. Please set the carbon.home";
                    throw new FileNotFoundException(msg);
                }
            }
            
            StAXOMBuilder builder = new StAXOMBuilder(inStream);
            return builder.getDocumentElement();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (XMLStreamException e) {
            e.printStackTrace();
        }
        
        return null;
    }

    protected void initializeDatabase(DataSource dataSource, String dialect) throws Exception {

	String value = System.getProperty("setup");
	if(value != null){
	    DBCreatorFactory factory = new DBCreatorFactory();
	    DatabaseCreator creator = factory.getDatabaseCreator(dialect, dataSource);
	    creator.createDefaultDatabaseTables();
	    creator.createHybridDatabaseTables();
	}
     //  only derby for now
     //  UMDatabaseManager manager = new UMDatabaseManager(dataSource);
     //  manager.createManagementTables();

    }
    
    public void addInitialData(OMElement parentElement, UserRealm defRealm) throws Exception {
        OMElement roleElem = parentElement.getFirstChildWithName(new QName("AdminRole"));
      
        if (roleElem == null) {
            throw new Exception("System cannot startup without system admin role.");
        }

        String adminRole = roleElem.getText();
        UserStoreAdmin usadmin = defRealm.getUserStoreAdmin();
        if (!usadmin.isExistingRole(adminRole)) {
            usadmin.addRole(adminRole);
        }
        
        OMElement adminElem = parentElement.getFirstChildWithName(new QName("AdminUser"));
        //system should be able to start without the admin user - mashup server
        String userName = adminElem.getFirstChildWithName(new QName("UserName")).getText();
        String password = adminElem.getFirstChildWithName(new QName("Password")).getText();
        if(adminElem != null){
            if(!usadmin.isExistingUser(userName)){
                usadmin.addUser(userName, password);
                usadmin.addUserToRole(userName, adminRole);
                if(!usadmin.isExistingRole("everyone"))
                    usadmin.addRole("everyone");
                
                usadmin.addUserToRole(userName, "everyone");
            }
        }

        List<String> sysConfig = new ArrayList<String>();
        OMElement sysPermissions = parentElement
                .getFirstChildWithName(new QName("SystemPermission"));
        if (sysPermissions == null) {
            throw new Exception("System cannot startup without system permissions.");
        }

        AccessControlAdmin acadmin = defRealm.getAccessControlAdmin();
        Iterator ite = sysPermissions.getChildElements();
        while (ite.hasNext()) {
            OMElement element = (OMElement) ite.next();
            String action = element.getText();
            SystemPermissions.addSystemPermission(action);
            sysConfig.add(action);
            if (!acadmin.isRoleAuthorized(adminRole, "System", action)) {
                acadmin.authorizeRole(adminRole, "System", action);
            }

        }

    
    }
    
    public UserRealm exposeDefaultRealmAsService(BundleContext bc, String url, String drivername, 
                                                String username, String password) throws Exception{
        UserRealm defRealm = new DefaultRealm();
        DefaultRealmConfig config = new DefaultRealmConfig();
        BasicDataSource dataSource = new BasicDataSource();
        dataSource.setUrl(url);
        dataSource.setDriverClassName(drivername);
        dataSource.setUsername(username);
        dataSource.setPassword(password);
        
        config.setConnectionURL(url);
        config.setDriverName(drivername);
        config.setConnectionUserName(username);
        config.setConnectionPassword(password);
        
        config.setDataSource(dataSource);
        defRealm.init(config);
        Dictionary dictionary = new Hashtable();
        dictionary.put(UserCoreConstants.REALM_GENRE, UserCoreConstants.DEFAULT_REALM);
        defaultRealmReference
          = bc.registerService(UserRealm.class.getName(), defRealm, dictionary);
        return defRealm;
    }
    
    

}
